#include <string.h>
#include <stdio.h>

#include "../include/stubs.h"
#include "../include/ioctl.h"
#include "../include/salayer.h"
#include "../include/pcicmd.h"

#include "../include/ifc_ldev.h"
#include "../include/ldevbase.h"
#include "../include/lcpi.h"


// LCPI BDMA with  PLX9050 PCI chip /////////////////////////////////////////////////
FDF(ULONG) DaqLCPI::GetWord_DM(USHORT Addr, PUSHORT Data)
{
   ULONG cbRet;
   USHORT par = Addr;
	ULONG ret;
   ret = sal->LDeviceIoControl( hVxd, sal->CTL(DIOC_GET_DM_A),//DIOC_GET_DM_W,
                           &par, sizeof(par),
                           Data, sizeof(USHORT),
                           &cbRet, L_STREAM_NULL
                         );
	return ((cbRet != (sizeof(USHORT))) ? L_ERROR_TIMEOUT:ret);
}

FDF(ULONG) DaqLCPI::PutWord_DM(USHORT Addr, USHORT Data)
{
   ULONG cbRet;
   USHORT par = Addr;
   return sal->LDeviceIoControl( hVxd, sal->CTL(DIOC_PUT_DM_A),//DIOC_PUT_DM_W,
                           &par, sizeof(par),
                           &Data, sizeof(USHORT),
                           &cbRet, L_STREAM_NULL
                         );
}

FDF(ULONG) DaqLCPI::SendCommand(USHORT Cmd)
{
   ULONG cbRet;
   USHORT data=0;
   USHORT par = Cmd;
   return sal->LDeviceIoControl( hVxd, sal->CTL(DIOC_COMMAND_PLX),
                           &par, sizeof(par),
                           &data, sizeof(USHORT),
                           &cbRet, L_STREAM_NULL
                         );
}

FDF(ULONG) DaqLCPI::PutArray_DM(USHORT Addr, ULONG Count, PUSHORT Data)
{
   ULONG cbRet;
   USHORT par = Addr;
   ULONG len=1024;
   int status;
   do
   {
      if(Count<len) len=Count;
      status = sal->LDeviceIoControl( hVxd, sal->CTL(DIOC_PUT_DM_A),
                           &par, sizeof(par),
                           Data, len*sizeof(USHORT),
                           &cbRet, L_STREAM_NULL
                         );
      if(status==L_ERROR) break;
      Data+=len;
//      par+=(USHORT)len;
      Count-=len;
   } while(Count);
   return status;
}

FDF(ULONG) DaqLCPI::GetArray_DM(USHORT Addr, ULONG Count, PUSHORT Data)
{
   ULONG cbRet;
   USHORT par = Addr;
   ULONG len=1024;
   int status;
   do
   {
      if(Count<len) len=Count;
      status = sal->LDeviceIoControl( hVxd, sal->CTL(DIOC_GET_DM_A),
                           &par, sizeof(par),
                           Data, len*sizeof(USHORT),
                           &cbRet, L_STREAM_NULL
                         );
      if(status==L_ERROR) break;
		if(cbRet != len*sizeof(USHORT)) { status = L_ERROR_TIMEOUT; break; }
      Data+=len;
//      par+=(USHORT)len;
      Count-=len;
   } while(Count);
   return status;
}

#define cmTEST_LCPI (12)

FDF(ULONG) DaqLCPI::PlataTest()
{
   USHORT d1;
   if(SendCommand(cmTEST_LCPI)) return L_ERROR;
   if(GetWord_DM(0x0000, &d1)) return L_ERROR;
   if(d1 != 0x5566) return L_ERROR;
   return L_SUCCESS;
}


ULONG DaqLCPI::FillADCparameters(PDAQ_PAR sp)
{

   PADC_PAR ap = (PADC_PAR)sp;

   if(ap->t1.FIFO==0) return L_ERROR;
   if(ap->t1.Pages==0) return L_ERROR;
   if(ap->t1.IrqStep==0) return L_ERROR;

   adc_par.t1.Scale = 0;

// More
   adc_par.t1.s_Type = ap->t1.s_Type;
   adc_par.t1.SynchroType=ap->t1.SynchroType;
   adc_par.t1.SynchroSensitivity=ap->t1.SynchroSensitivity;
   adc_par.t1.SynchroMode=ap->t1.SynchroMode;
   adc_par.t1.AdChannel=ap->t1.AdChannel;
   adc_par.t1.AdPorog=ap->t1.AdPorog;

   adc_par.t1.FIFO=ap->t1.FIFO;
   adc_par.t1.IrqStep = ap->t1.IrqStep;
   adc_par.t1.Pages = ap->t1.Pages;
   if(ap->t1.NCh>128) ap->t1.NCh=128;
   adc_par.t1.NCh=ap->t1.NCh;
   for(USHORT i=0; i<ap->t1.NCh; i++) adc_par.t1.Chn[i]=ap->t1.Chn[i];

   adc_par.t1.AutoInit = ap->t1.AutoInit;
   adc_par.t1.IrqEna=ap->t1.IrqEna;
   adc_par.t1.AdcEna=ap->t1.AdcEna;

// make a copy  of adc_par in wadc_par for C-style interface to driver ////////
   CopyDAQtoWDAQ(&adc_par.t1,&wadc_par.t3,2);
   return L_SUCCESS;
}



ULONG DaqLCPI::csGetParameter(ULONG name, PULONG param, ULONG status)
{
   switch(name)
   {
      case L_POINT_SIZE: *param = 2; break;
      default: return L_NOTSUPPORTED;
   }
   return status;
}

ULONG DaqLCPI::csSetParameter(ULONG name, PULONG param, ULONG status)
{
   switch(name)
   {
      case 9999:
      default: return L_NOTSUPPORTED;
   }
   return status;
}
