#include "USB_USBMain.h"
#include "USBBackground.h"
#include "bootrom.h"
#include "flash.h"
#include "timers.h"
#include "pio.h"
#include "main.h"
#include "cpld.h"

/*================================================================================================*/
#define BOOT_JUMPER_NORMAL()    ((AT91C_BASE_PIOA->PIO_PDSR & PIO_ALTBOOT) == 0)
#define USB_DESC_SECTION_ADDR   0x00104100  /*   USB     */
/*================================================================================================*/

/*================================================================================================*/
unsigned int g_MainFlag = 0;
int g_FW_Valid = 0;
const t_FLASH_Metadata *g_FW_pMeta;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
static __noreturn void f_RunMainFirmware
    (void)
    {
    __disable_interrupt();
    /*  REMAP */
    AT91C_BASE_MC->MC_RCR = AT91C_MC_RCB;
    ((void(*)(void))FLASH_FIRMWARE_START)();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int SetLowPowerMode                 /*     */
    (
    int LowPower                    /* ()  -   (1)   (0)  */
    )                               /*  1  , 0   */
    {
    if (LowPower)
        { /* Enter low power mode */
        /*   */
        CPLD_PowerOff();
        }
    else
        { /* Leave low power mode */
        /*   ,     (   ) */
        if (CPLD_PowerGood())
            return 1;
        /*   */
        if (!CPLD_PowerOn(CPLD_POWER_ON_TIMEOUT))
            return 0;
        }
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void main()
    {
    int SoftStart;
    int have_rq;

    /*       ,   f_SoftStart */
    SoftStart = (BOOTROM_GetCommand() == BOOTCMD_ENTER_BOOTROM);
    have_rq = DeviceConfigured() && BOOTROM_GetRequest(&CurrentUSBRequest);
    /*      */
    BOOTROM_Clear();

    TIMER_Init();

    /*        */
    g_FW_Valid = FLASH_CheckFirmwareState(&g_FW_pMeta);
    /*   ,         
        ,      */
    if (g_FW_Valid && (g_FW_pMeta->FirmwareMode & FLASH_FWMODE_FASTRUN)
        && !SoftStart && BOOT_JUMPER_NORMAL())
        {
        f_RunMainFirmware();
        }

    /*   ,       USB  */
    if (g_FW_Valid && BOOT_JUMPER_NORMAL())
        USB_LoadDescriptorDirectory((void*)USB_DESC_SECTION_ADDR);

    /*  USB */
    if (!have_rq)
        { /* no main, USB not configured */
        ConnectUSB();
        while (!USBConfigured)
            {
            USBBG_Worker(); /*   watchdog */
            }
        }
    else
        { /* after main, USB already configured */
        ConnectUSBFromBoot();
        USBConfigured = true;
        g_TIMER_BlinkFlag = 1;
        }

    /*     USB */
    SetLowPowerMode(0);

    /*    usb device request,    */
    if (have_rq)
        {
        if (CurrentUSBRequest.bRequest < VDRQ_COUNT)
            {
            __disable_interrupt();
            VendorDeviceRequest[CurrentUSBRequest.bRequest]();
            __enable_interrupt();
            }
        }

    /*   */
    for(;;)
        {
        USBBG_Worker(); /*   watchdog */
        if (g_MainFlag & MAIN_RUN_USER_FIRMWARE)
            {
            g_MainFlag &= ~MAIN_RUN_USER_FIRMWARE;
            if (BOOT_JUMPER_NORMAL() && g_FW_Valid)
                {
                BOOTROM_SetRequest(&CurrentUSBRequest);
                f_RunMainFirmware();
                }
            else
                {
                USBBG_RequestError();
                }
            }

#if MAIN_SLEEP_ON_USB_SUSPEND
        if (g_MainFlag & MAIN_ENTER_SLEEP_MODE)
            {
            g_MainFlag &= ~MAIN_ENTER_SLEEP_MODE;
            SetLowPowerMode(1);
            }

        if (g_MainFlag & MAIN_EXIT_SLEEP_MODE)
            {
            g_MainFlag &= ~MAIN_EXIT_SLEEP_MODE;
            SetLowPowerMode(0);
            }
#endif
        } /* for (;;) */
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
