/*================================================================================================*
 *  
 * 1.   (PIT) -   (  )
 * 2. -   (TCLK0 -> TIOA2)   TC2
 * 3.     ( )   TC1
 *================================================================================================*/

#include "timers.h"
#include "pio.h"
#include "HAL.h"

/*================================================================================================*/
#define PIT_TICK_MS 50                          /*  PIT,  */
#define PIT_PERIOD  ((CLK_MCK_FREQ / 16000) * PIT_TICK_MS)  /*  PIT () */
#if ((PIT_PERIOD < 1) || (PIT_PERIOD > 0x100000))
#error "PIT period out of range."
#endif

#define LED_ENABLED     1

#define LED_TOGGLE_MS   100     /*    ,  */
/*================================================================================================*/

/*================================================================================================*/
int g_TIMER_BlinkFlag = 0;
unsigned long g_TIMER_PseudoRandom = 88675123;
#if LED_ENABLED
static unsigned int f_PIT_Elapsed = 0;
static int f_LED_BlinkStage = 0;
static int f_LED = 0; /*    */
static AT91_REG *f_PIOREG[2] =
    { &AT91C_BASE_PIOA->PIO_CODR, &AT91C_BASE_PIOA->PIO_SODR };
#define SET_LED(st)     *f_PIOREG[f_LED = st] = PIO_LED
#endif
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
__arm __ramfunc void System_Interrupt_Handler
    (void)
    {
    /*     */
    if (AT91C_BASE_PITC->PITC_PISR & AT91C_PITC_PITS)
        {
        volatile unsigned int elapsed_ticks = (AT91C_BASE_PITC->PITC_PIVR >> 20);

        /* Xorshift pseudo-RNG, http://en.wikipedia.org/wiki/Xorshift */
        static unsigned long x = 123456789, y = 362436069, z = 521288629;
        unsigned long t;
        t = x ^ (x << 11);
        x = y; y = z; z = g_TIMER_PseudoRandom;
        g_TIMER_PseudoRandom =
            (g_TIMER_PseudoRandom ^ (g_TIMER_PseudoRandom >> 19)) ^ (t ^ (t >> 8));

#if LED_ENABLED
        f_PIT_Elapsed += elapsed_ticks;
        /*  :      ,  
              ",  USB configured" */
        if (f_LED_BlinkStage == 0)
            {
            if (g_TIMER_BlinkFlag)
                { /*    */
                g_TIMER_BlinkFlag = 0;
                f_LED_BlinkStage = 1;
                f_PIT_Elapsed = 0;
                SET_LED(f_LED ^ 1);
                }
            }
        else /* f_LED_BlinkStage != 0 */
            {
            if (f_PIT_Elapsed >= (LED_TOGGLE_MS / PIT_TICK_MS))
                {
                if (f_LED_BlinkStage == 1)
                    {
                    SET_LED(f_LED ^ 1);
                    f_LED_BlinkStage = 2;
                    }
                else
                    {
                    SET_LED(USBConfigured && !USBSuspend);
                    f_LED_BlinkStage = 0;
                    }
                f_PIT_Elapsed = 0;
                }
            }
#endif
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void TIMER_Init                     /*     PIT */
    (void)
    {
    /*  PIT */
    SetInterrupt(AT91C_ID_SYS, System_Interrupt_Handler, PRI_LOW, INTERNAL_HIGH_LEVEL);
    EnableInterruptsMask(1 << AT91C_ID_SYS);
    if (AT91C_BASE_PITC->PITC_PIVR); /* Clear PIT counters and interrupts */
    AT91C_BASE_PITC->PITC_PIMR =
        (AT91C_PITC_PIV & (PIT_PERIOD - 1)) |
        AT91C_PITC_PITEN |          /* enable PIT */
        AT91C_PITC_PITIEN;          /* enable interrupt */

    /*   TC   */
    AT91C_BASE_TC0->TC_CCR = AT91C_TC_CLKDIS;
    AT91C_BASE_TC1->TC_CCR = AT91C_TC_CLKDIS;
    AT91C_BASE_TC2->TC_CCR = AT91C_TC_CLKDIS;

    /*       */
    /*      */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_TC1);  /*  TC1 clock */
    AT91C_BASE_TC1->TC_IDR = 0xFF;      /* Disable all interrupts */
    AT91C_BASE_TC1->TC_CMR =
        TC1_CLK_SELECT |                /* Clock source */
        (AT91C_TC_CLKI & 0) |           /* No invert (increment on rising edge) */
        AT91C_TC_BURST_NONE |           /* Clock ungated */
        AT91C_TC_CPCSTOP |              /* Stop clock on RC */
        AT91C_TC_CPCDIS |               /* Disable clock on RC */
        AT91C_TC_EEVTEDG_NONE |         /* No external event */
        AT91C_TC_EEVT_TIOB |
        (AT91C_TC_ENETRG & 0) |         /* No external event trigger */
        AT91C_TC_WAVESEL_UP |           /* Waveform: count up without reset on RC */
        AT91C_TC_WAVE |                 /* Waveform mode = 1 */
        AT91C_TC_ACPA_NONE |            /* RA compare effect on TIOA */
        AT91C_TC_ACPC_NONE |            /* RC compare effect on TIOA */
        AT91C_TC_AEEVT_NONE |           /* External event effect on TIOA */
        AT91C_TC_ASWTRG_NONE |          /* Software trigger (start) effect on TIOA */
        AT91C_TC_BCPB_NONE |            /* no                */
        AT91C_TC_BCPC_NONE |            /*    output         */
        AT91C_TC_BEEVT_NONE |           /*           on      */
        AT91C_TC_BSWTRG_NONE;           /*              TIOB */
    AT91C_BASE_TC1->TC_RC = 0;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
