#ifndef ADVINTERRUPT_H_
#define ADVINTERRUPT_H_

enum InterruptLevelType
{
  INTERNAL_HIGH_LEVEL=	                AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL,
  EXTERNAL_LOW_LEVEL= 	                AT91C_AIC_SRCTYPE_EXT_LOW_LEVEL,
  INTERNAL_POSITIVE_EDGE= 	            AT91C_AIC_SRCTYPE_INT_POSITIVE_EDGE,
  EXTERNAL_NEGATIVE_EDGE= 	            AT91C_AIC_SRCTYPE_EXT_NEGATIVE_EDGE,
  HIGH_LEVEL=                           AT91C_AIC_SRCTYPE_HIGH_LEVEL,
  INTERNAL_EXTERNAL_POSITIVE_EDGE=      AT91C_AIC_SRCTYPE_POSITIVE_EDGE
};

enum IRQ_Priority_Level
{
  PRI_LOWEST = 0,
  PRI_LOW,
  PRI_BELOW_NORMAL,
  PRI_NORMAL,
  PRI_ABOVE_NORMAL,
  PRI_HIGH,
  PRI_HIGHEST
};


#define SetInterrupt(PeripheralID,Handler, Priority, ThisType) AT91F_AIC_ConfigureIt(AT91C_BASE_AIC,PeripheralID,Priority, ThisType, Handler)

#define EnableInterruptsMask(Mask)  AT91C_BASE_AIC->AIC_IECR=Mask
#define DisableInterruptsMask(Mask)  AT91C_BASE_AIC->AIC_IDCR=Mask

#define EnableFastForcingInterrupt(PeripheralID) AT91C_BASE_AIC->AIC_FFER = 0x1 << PeripheralID
#define DisableFastForcingInterrupt(PeripheralID) AT91C_BASE_AIC->AIC_FFDR = 0x1 << PeripheralID

#define  EnableInterrupt(PeripheralID) AT91C_BASE_AIC->AIC_IECR = (uint)(0x1 << PeripheralID)
#define  DisableInterrupt(PeripheralID) AT91C_BASE_AIC->AIC_IDCR = (uint)(0x1 << PeripheralID)

#define EnableInterrupts() AT91C_BASE_AIC->AIC_DCR = 0
#define DisableInterrupts() AT91C_BASE_AIC->AIC_DCR = 1<<1

#define SuppressCallInterrupt(PeripheralID) AT91C_BASE_AIC->AIC_ISCR=(0x1<<PeripheralID)

#define ClearInterrupt(PeripheralID)  AT91C_BASE_AIC->AIC_ICCR = (uint)(0x1<<PeripheralID)

inline void DisableInterrupt_And_Clear(uint PeripheralID)
{
   uint mask = 0x1 << PeripheralID;
   //* Disable the interrupt on the interrupt controller
   AT91C_BASE_AIC->AIC_IDCR = mask ;
    //* Clear the interrupt on the Interrupt Controller ( if one is pending )
   ClearInterrupt(mask);
}

inline void ChangeInterruptSourceType(uint PeripheralID, uint SourceType)
{   
    //* Disable the interrupt on the interrupt controller
    DisableInterrupt(PeripheralID);
    
    AT91C_BASE_AIC->AIC_SMR[PeripheralID]&=~AT91C_AIC_SRCTYPE;
    AT91C_BASE_AIC->AIC_SMR[PeripheralID]|=(SourceType&AT91C_AIC_SRCTYPE);
    
    ClearInterrupt(PeripheralID);
}

#endif /* ADVINTERRUPT_H_ */
