#ifndef USB_H_
#define USB_H_

#include "HAL.h"

#define DEFAULT_CONTROL_ENDPOINT_SIZE 8
#define DEFAULT_CONTROL_ENDPOINT 0

enum EndPointType
{
  CONTROL=0,
  ISOCHRONOUS_OUT=1,
  ISOCHRONOUS_IN=5,
  BULK_OUT=2,
  BULK_IN=6,
  INTERRUPT_OUT=3,
  INTERRUPT_IN=7
};

typedef struct _USBRequest
    {
    byte bmRequestType;
    byte bRequest;
    ushort wValue;
    ushort wIndex;
    ushort wLength;
    }
    USBRequest;

#define Udp_ep_clr_flag(Endpoint, flags) { \
	while (AT91C_BASE_UDP->UDP_CSR[Endpoint] & (flags)) \
	AT91C_BASE_UDP->UDP_CSR[Endpoint] &= ~(flags); \
}
//! Set flags of UDP UDP_CSR register and wait for synchronization
#define Udp_ep_set_flag(Endpoint, flags) { \
	while ((AT91C_BASE_UDP->UDP_CSR[Endpoint] & (flags)) != (flags)) \
	AT91C_BASE_UDP->UDP_CSR[Endpoint] |= (flags); \
}

#define DisableUSBInterrupt(Number)  AT91C_BASE_UDP->UDP_IDR=(1<<Number)

#define EnableUSBInterrupt(Number)   AT91C_BASE_UDP->UDP_IER=(1<<Number)

#define Enable_USB_Interrupts(Mask)  AT91C_BASE_UDP->UDP_IER=Mask

#define Disable_USB_Interrupts(Mask)  AT91C_BASE_UDP->UDP_IDR=Mask

#define Clear_Usb_Interrupt(InterruptNumber)  AT91C_BASE_UDP->UDP_ICR=(1<<InterruptNumber)

#define Clear_Usb_Interrupts(Mask)  AT91C_BASE_UDP->UDP_ICR=Mask

#define GetFrameNumbers() AT91C_BASE_UDP->UDP_NUM&AT91C_UDP_FRM_NUM

#define ResetEndpoints(Mask) AT91C_BASE_UDP->UDP_RSTEP=Mask;AT91C_BASE_UDP->UDP_RSTEP=0;AT91C_BASE_UDP->UDP_FADDR =AT91C_BASE_UDP->UDP_FADDR| AT91C_UDP_FEN

#define SetAddress(Address)   AT91C_BASE_UDP->UDP_FADDR&=~AT91C_UDP_FADD; AT91C_BASE_UDP->UDP_FADDR|=Address;

/*
  
*/
#define ConfigureEndpoint(EndPointNumber, Type, Enable) { \
    while ((AT91C_BASE_UDP->UDP_CSR[EndPointNumber] & (AT91C_UDP_EPTYPE | AT91C_UDP_EPEDS)) \
        != ((((Type) << 8) & AT91C_UDP_EPTYPE) | (((Enable) << 15) & AT91C_UDP_EPEDS)))     \
        AT91C_BASE_UDP->UDP_CSR[EndPointNumber] =                                           \
            ((((Type) << 8) & AT91C_UDP_EPTYPE) | (((Enable) << 15) & AT91C_UDP_EPEDS));    \
    }

#define EnableEndpoint(EndPointNumber)  Udp_ep_set_flag(EndPointNumber, AT91C_UDP_EPEDS)

#define DisableEndpoint(EndPointNumber) Udp_ep_clr_flag(EndPointNumber, AT91C_UDP_EPEDS)

#define EndpointEnabled(EndPointNumber)  ((AT91C_BASE_UDP->UDP_CSR[EndPointNumber]>>15)&1)

inline void SetEndpointStall(int EndPointNumber, bool Stall)
    {
    if(Stall)
        {
        Udp_ep_set_flag(EndPointNumber, AT91C_UDP_FORCESTALL);
        }
    else
        {
        Udp_ep_clr_flag(EndPointNumber, (AT91C_UDP_FORCESTALL | AT91C_UDP_ISOERROR));
        }
    }

inline bool EndPointStalled(int EndPointNumber)
{
  return ((AT91C_BASE_UDP->UDP_CSR[EndPointNumber]&AT91C_UDP_ISOERROR)>>3)&1;
}

#define GetNumberBytesFIFO(EndPointNumber)  ((AT91C_BASE_UDP->UDP_CSR[EndPointNumber]>>16)&0x7ff)

/*
   
*/
inline bool TestFIFOREadyToWrite(byte EndPointNumber)
{
  return (AT91C_BASE_UDP->UDP_CSR[EndPointNumber]>>4)&1;
}

/*
  
*/

inline void USBSetAddress(ushort Address)
{
  AT91C_BASE_UDP->UDP_FADDR = (AT91C_UDP_FEN | Address);
  AT91C_BASE_UDP->UDP_GLBSTATE  = (Address) ? AT91C_UDP_FADDEN : 0;
}

#define DeviceConfigured() ((AT91C_BASE_UDP->UDP_GLBSTATE>>1)&1)

#define SetGlobalStateConfig()  AT91C_BASE_UDP->UDP_GLBSTATE|=AT91C_UDP_CONFG

#define ClrGlobalStateConfig()  AT91C_BASE_UDP->UDP_GLBSTATE&=~AT91C_UDP_CONFG

extern volatile bool USBConfigured;
extern volatile bool USBSuspend;
extern volatile int USB_Send_Bank_Full;
extern volatile bool CurrentBank0;

#endif /* USB_H_ */
