#ifndef USB_STACK_H_
#define USB_STACK_H_

#define REQUEST_ANSWER_DIRECTION 0x80
#define REQUEST_ANSWER_DIRECTION_HOST_TO_DEVICE (0x0)
#define REQUEST_ANSWER_DIRECTION_DEVICE_TO_HOST (0x80)


#define REQUEST_TYPE 0x60
#define REQUEST_TYPE_STANDARD 0
#define REQUEST_TYPE_CLASS 0x20
#define REQUEST_TYPE_VENDOR 0x40

#define REQUEST_RECIPIENT (0x1f)
#define REQUEST_RECIPIENT_DEVICE (0x0)
#define REQUEST_RECIPIENT_INTERFACE (0x1)
#define REQUEST_RECIPIENT_ENDPOINT (0x2)
#define REQUEST_RECIPIENT_OTHER (0x3)


#define FW_USB_REQUEST_MASK         0xF
#define FW_DEVICE_ADDRESS_MASK      0x7F

//    request
#define DEVICE_OUT    0x00
#define INTERFACE_OUT 0x01
#define ENDPOINT_OUT  0x02
#define OTHER_OUT     0x03

//    request
#define DEVICE_IN    0x80
#define INTERFACE_IN 0x81
#define ENDPOINT_IN  0x82
#define OTHER_IN     0x83

// Feature Selector

#define FEATURE_DEVICE_REMOTE_WAKEUP 1
#define FEATURE_ENDPOINT_HALT 0
#define FEATURE_TEST_MODE 2



extern USBRequest CurrentUSBRequest;


void   std_getStatus(void);        /*  0 */
void   std_clearFeature(void);     /*  1 */
void   std_reserved(void);         /*  2 */
void   std_setFeature(void);       /*  3 */
void   std_reserved(void);         /*  4 */
void   std_setAddress(void);       /*  5 */
void   std_getDescriptor(void);    /*  6 */
void   std_setDescriptor(void);    /*  7 */
void   std_getConfiguration(void); /*  8 */
void   std_setConfiguration(void); /*  9 */
void   std_getInterface(void);     /* 10 */
void   std_setInterface(void);     /* 11 */
void   std_synch_frame(void);       /* 12 */


__arm __ramfunc void ControlCommandProcessing(void);
void EndBusResProcessing(void);
void EnterSleepMode(void);
void ExitSleepMode(void);

#endif /* USB_STACK_H_ */
