/*================================================================================================*
 *      Altera
 * (USART0, ,  SCK (8 ), 9bitTX/8bitRX, MSB first, 1 stop, no parity)
 *================================================================================================*/

#ifndef CPLD_H_
#define CPLD_H_

/*================================================================================================*/
/*     CPLD_SetADCMode() */
#define CPLD_ADCMODE_VALID_BITS             0xF3

#define CPLD_ADCMODE_SAMPLESYNC_MASK        0x03 /*   : */
#define CPLD_ADCMODE_SAMPLESYNC_INTERNAL    0x00 /*         SYNC */
#define CPLD_ADCMODE_SAMPLESYNC_MASTER      0x01 /*         SYNC (. ) */
#define CPLD_ADCMODE_SAMPLESYNC_SLAVE_RISE  0x02 /*         INT (.) */
#define CPLD_ADCMODE_SAMPLESYNC_SLAVE_FALL  0x03 /*         INT */

#define CPLD_ADCMODE_STARTSYNC_MASK         0x30 /*    : */
#define CPLD_ADCMODE_STARTSYNC_SOFT         0x00 /*        START ADC */
#define CPLD_ADCMODE_STARTSYNC_ANALOG       0x10 /*       */
#define CPLD_ADCMODE_STARTSYNC_EXT_RISE     0x20 /*         INT */
#define CPLD_ADCMODE_STARTSYNC_EXT_FALL     0x30 /*         INT (.) */

#define CPLD_ADCMODE_ANALOG_MASK            0xC0 /*   : */
#define CPLD_ADCMODE_ANALOG_LVL_ABOVE       0x00 /*      ,  */
#define CPLD_ADCMODE_ANALOG_LVL_BELOW       0x40 /*      ,  */
#define CPLD_ADCMODE_ANALOG_EDGE_RISE       0x80 /*       ,  */
#define CPLD_ADCMODE_ANALOG_EDGE_FALL       0xC0 /*       ,  */

/*     CPLD_StartADC() */
#define CPLD_ADCSTART_STOP                  0x00 /*  */
#define CPLD_ADCSTART_CONTINUOUS            0x01 /*     */
#define CPLD_ADCSTART_FRAMESYNC             0x03 /*   (  ) */
#define CPLD_ADCSTART_SINGLE_FRAME          0x07 /*      */

/*   */
#define CPLD_CONFIG_RESET_STATE             0x00 /*  .    */
#define CPLD_CONFIG_VALID_BITS              0x07 /*    */
#define CPLD_CONFIG_TTLOUT_ENABLE           0x01 /*   TTL  */
#define CPLD_CONFIG_DAC_ENABLE              0x02 /*    (  ) */
#define CPLD_CONFIG_SPI_MASTER_ENABLE       0x04 /*    SPI  */

/*   */
#define CPLD_DACCTL_ASYNC_PRELOAD           0x00 /*     */
#define CPLD_DACCTL_ASYNC_OUTPUT            0x01 /*       . */

/*      */
#define CPLD_DAC_STOP                       0x00 /*      */
#define CPLD_DAC_START                      0x01 /*      */
#define CPLD_DAC_SYNC_WITH_ADC              0x80 /*     (  ) */
#define CPLD_DAC_FREQDIV_MASK               0x0E /*    */
#define CPLD_DAC_FREQDIV_SHIFT              1    /*    */

/* ,  ( 1000) */
#define CPLD_POWER_ON_TIMEOUT               1000
#define CPLD_COMMAND_TIMEOUT                100
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void CPLD_InitUSART                 /*     USART */
    (void);
/*------------------------------------------------------------------------------------------------*/
int CPLD_PowerGood                  /*        */
    (void);                         /*  1 =   ; 0 =    */
/*------------------------------------------------------------------------------------------------*/
int CPLD_PowerOn                    /*        */
    (
    unsigned int Timeout            /* ()  -    ( ~1000, 0 = ) */
    );                              /*  ,  0   */
/*------------------------------------------------------------------------------------------------*/
void CPLD_PowerOff                  /*        */
    (void);
/*------------------------------------------------------------------------------------------------*/
void CPLD_SetTimeout                /*      */
    (
    unsigned int Timeout            /* ()  -    ( ~1000, 0 = ) */
    );
/*------------------------------------------------------------------------------------------------*/
int CPLD_RunGenericCommand          /*     () */
    (
    unsigned int CommandLen,        /* ()  -      */
    unsigned char *CommandBuffer,   /* ()  -      */
    unsigned int ReplyLen,          /* ()  -    */
    unsigned char *ReplyBuffer      /* () -       */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
/*  :        .. */
/*------------------------------------------------------------------------------------------------*/
unsigned int CPLD_CFG_GetLastConfig /*       */
    (void);                         /*    ( CPLD_CONFIG_...) */
/*------------------------------------------------------------------------------------------------*/
int CPLD_CFG_SetConfigBits          /*     */
    (
    unsigned int RegMask,           /* ()  -    ( CPLD_CONFIG_...) */
    unsigned int RegValue           /* ()  -    */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_InitAndPreload         /*    */
    (
    unsigned int Mode               /* ()  -  (  CPLD_ADCMODE_...) */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
unsigned int CPLD_ADC_GetLastMode   /*      */
    (void);                         /*   (  CPLD_ADCMODE_...) */
/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetFrameDelay          /*     */
    (
    unsigned int CycleCount         /* ()  -    , 0..255 */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetSamplingRate        /*     */
    (
    unsigned int FreqDiv            /* ()  -  , 16  (F = 8000000/(DIV+1)) */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetupAnalogSync        /*    */
    (
    unsigned int ChannelSelect,     /* ()  -   ( ) */
    int ThresholdValue              /* ()  -   */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_Start                  /*   */
    (
    unsigned int Mode               /*   (CPLD_ADCSTART_...) */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_Stop                   /*   */
    (void);                         /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Async                  /*      */
    (
    unsigned int DAC_Command        /* ()  -  (CPLD_DACCTL_ASYNC_...) */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Start                  /*   */
    (
    unsigned int SyncWithADC,       /* 0 =  , 1 =      */
    unsigned int FreqDiv            /*   (0..7, F=200kHz/(N+1)) */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Stop                   /*   */
    (void);                         /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_TTL_In                     /*    DI 0..15 */
    (
    unsigned short int *pValue      /* () -    -  */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int CPLD_TTL_Out                    /*    DOUT 0..15 */
    (
    unsigned short int Value        /* ()  -   (16 ) */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /* CPLD_H_ */
