/*================================================================================================*
 *  Watchdog
 *================================================================================================*/

#ifndef WDT_H_
#define WDT_H_

#include "chipio.h"

/*================================================================================================*/
/*  watchdog   SLOWCLK/128, ..  250 Hz */
#define WDT_WDV     0x0FF       /*  WDT */
#define WDT_WDD     0xFFF       /*      WDT */
#define WDT_KEY     0xA5000000
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
inline void WDT_Init                /*  watchdog (   ) */
    (void)
    {
    AT91C_BASE_WDTC->WDTC_WDMR =
        (AT91C_WDTC_WDV & (WDT_WDV << 0)) |     /* Counter value */
        (AT91C_WDTC_WDRSTEN) |                  /* Generate reset on timeout */
        (AT91C_WDTC_WDRPROC & 0) |              /* Only reset core (0 = reset entire chip) */
        (AT91C_WDTC_WDD & (WDT_WDD << 16)) |    /* Delta value */
        (AT91C_WDTC_WDDBGHLT) |                 /* Watchdog Debug Halt */
        (AT91C_WDTC_WDIDLEHLT);                 /* Watchdog Idle Halt */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
inline void WDT_Disable             /*  watchdog ( ) */
    (void)
    {
    AT91C_BASE_WDTC->WDTC_WDMR = AT91C_WDTC_WDDIS;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
inline void WDT_Restart             /*   watchdog */
    (void)
    {
    AT91C_BASE_WDTC->WDTC_WDCR = WDT_KEY | AT91C_WDTC_WDRSTT;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /* WDT_H_ */
