#include "USB_USBMain.h"
#include "Configure_USB.h"

/*================================================================================================*/
#define USB_DESCDIR_SIGN    0x31444455 /* "UDD1" */

/*  Eng/Rus    (CP1251)  unicode */
#define U(ch)               (((ch) < 0xC0) ? (ch) : 0x410 + (ch) - 0xC0)

#define USB_STR_HEADER(len)     ((USB_STRING_DESCRIPTOR_TYPE << 8) | (((len) + 2) & 0xFF))
#define DECLARE_USB_STRING(var_name, array_def)   \
    static const ushort _tmp_##var_name[] = { array_def }; \
    static const ushort var_name[] = \
        { USB_STR_HEADER(sizeof(_tmp_##var_name)), array_def }
/*================================================================================================*/

/*================================================================================================*/
static const USB_DEVICE_DESCRIPTOR f_DeviceDescriptor =
    {
    sizeof(USB_DEVICE_DESCRIPTOR),      //bLength
    USB_DEVICE_DESCRIPTOR_TYPE,         //bDescriptorType
    0x0110,                             //bcdUSB - 1.
    0x00,                               //bDeviceClass
    0x00,                               //bDeviceSubClass
    0x00,                               //bDeviceProtocol
    0x08,                               //bMaxPacketSize0 -   
    DEVICE_VID,                         //idVendor - LCARD
    DEVICE_PID,                         //idProduct
    1,                                  //bcdDevice
    0x01,                               //iManufacturer
    0x02,                               //iProduct
    0x00,                               //iSerialNumber --    3
    0x01                                //bNumConfigurations
    };


#pragma diag_suppress = Pe550 /*    "set but never used" */
static const ushort f_UsbStrLangList[] =
    { USB_STR_HEADER(4), 0x409 /* English */, 0x419 /* Russian */};

DECLARE_USB_STRING(f_UsbStrManufacturerEng, USBSTR_MANUFACTURER_ENG);
DECLARE_USB_STRING(f_UsbStrProductNameEng,  USBSTR_PRODUCT_NAME_ENG);
DECLARE_USB_STRING(f_UsbStrConfigEng,       USBSTR_CONFIG_ENG);

DECLARE_USB_STRING(f_UsbStrManufacturerRus, USBSTR_MANUFACTURER_RUS);
DECLARE_USB_STRING(f_UsbStrProductNameRus,  USBSTR_PRODUCT_NAME_RUS);
DECLARE_USB_STRING(f_UsbStrConfigRus,       USBSTR_CONFIG_RUS);

DECLARE_USB_STRING(f_UsbStrSerialNumber,    USBSTR_SERIAL_NUMBER);
#pragma diag_default = Pe550

/*      */
static const ushort *const f_UsbStringList[] =
   /* 0x01                   0x02                    0x03                  0x04             */
    {
    f_UsbStrManufacturerEng, f_UsbStrProductNameEng, f_UsbStrSerialNumber, f_UsbStrConfigEng,
    f_UsbStrManufacturerRus, f_UsbStrProductNameRus, f_UsbStrSerialNumber, f_UsbStrConfigRus
    };

static const FULL_CONFIG_DESCRIPTOR f_ConfigurationDescriptor_Full=
    {
    // USB_CONFIGURATION_DESCRIPTOR g_ConfigurationDescriptor =
        {
        sizeof(USB_CONFIGURATION_DESCRIPTOR),   //bLength
        USB_CONFIGURATION_DESCRIPTOR_TYPE,      //bDescriptorType
        sizeof(USB_CONFIGURATION_DESCRIPTOR) +
            sizeof(USB_INTERFACE_DESCRIPTOR) +
            (sizeof(USB_ENDPOINT_DESCRIPTOR)*ENDPOINT_COUNT),      //wTotalLength
        0x01,                                   //bNumInterfaces
        0x01,                                   //bConfigurationValue
        0x04,                                   //iConfiguration
        USB_CONFIG_BUS_NOWAKEUP,               // attribute : Bus-powered, no wakeup
        CURRENT_CONSUMPTION_MA/2                  // Maximum power consumption (in 2mA unit, so 0x64 =100= 200mA)
        },

    //USB_INTERFACE_DESCRIPTOR g_InterfaceDescriptor =
        {
        sizeof(USB_INTERFACE_DESCRIPTOR),       //bLength
        USB_INTERFACE_DESCRIPTOR_TYPE,          //bDescriptorType
        0x00,                                   //bInterfaceNumber
        0x00,                                   //bAlternateSetting
        ENDPOINT_COUNT,                         //bNumEndpoints
        0x00,                                   //bInterfaceClass
        0x00,                                   //bInterfaceSubClass
        0x00,                                   //bInterfaceProtocol  
        0x00                                    //iInterface
        },

    //USB_ENDPOINT_DESCRIPTOR g_EndpointDescriptors[ENDPOINT_COUNT] =
        {
            {
            sizeof(USB_ENDPOINT_DESCRIPTOR),      //bLength
            USB_ENDPOINT_DESCRIPTOR_TYPE,         //bDescriptorType,
            OUT_ENDPOINT|USB_ENDPOINT_DIRECTION_OUT,       //bEndpointAddress
            USB_ENDPOINT_TYPE_BULK,               //bmAttributes
            0x0040,                               //wMaxPacketSize
            0x00                                  //bInterval
            },
            {
            sizeof(USB_ENDPOINT_DESCRIPTOR),      //bLength
            USB_ENDPOINT_DESCRIPTOR_TYPE,         //bDescriptorType,
            IN_ENDPOINT|USB_ENDPOINT_DIRECTION_IN,      //bEndpointAddress - LTR 10
            USB_ENDPOINT_TYPE_BULK,               //bmAttributes
            0x0040,                               //wMaxPacketSize
            0x00                                  //bInterval
            }
        }
    };

/*   USB  :
 * -   -   ;
 * -    -   
 *          
 * USB_LoadDescriptorDirectory() */
#pragma location = "USB_DESCDIR_SECTION"
static const t_USB_DescriptorDirectory f_DescDir =
    {
    USB_DESCDIR_SIGN,
    /*      */
        sizeof(f_UsbStringList)/sizeof(f_UsbStringList[0]) /
        (sizeof(f_UsbStrLangList)/sizeof(f_UsbStrLangList[0]) - 1),
    &f_DeviceDescriptor,
    &f_ConfigurationDescriptor_Full,
    f_UsbStrLangList,
    f_UsbStringList,
    };

const t_USB_DescriptorDirectory *g_DescDirPtr = &f_DescDir;
/*================================================================================================*/

/*================================================================================================*/
#define PTR_IN_FLASH(ptr) (((unsigned long)(ptr) >= (unsigned long)AT91C_IFLASH) && \
                           ((unsigned long)(ptr) < (unsigned long)AT91C_IFLASH + AT91C_IFLASH_SIZE))
/*------------------------------------------------------------------------------------------------*/
int USB_LoadDescriptorDirectory
    (
    const t_USB_DescriptorDirectory *pDir
    )
    {
    int k, i, lang_count;

    /*     */
    if (!pDir || (pDir->Signature != USB_DESCDIR_SIGN))
        return 0;
    /*      flash , ..     
         */
    if (!PTR_IN_FLASH(pDir->DeviceDescriptorPtr))
        return 0;
    if (!PTR_IN_FLASH(pDir->ConfigDescriptorPtr))
        return 0;
    if (!PTR_IN_FLASH(pDir->StrLangList))
        return 0;
    if (!PTR_IN_FLASH(pDir->String))
        return 0;

    /*    :
     *  :
     *      PID  VID      
     *           ( )
     *         endpoint'  8,   64.
     *            1
     *          
     */

    /*   */
    if (pDir->DeviceDescriptorPtr->bLength != sizeof(USB_DEVICE_DESCRIPTOR))
        return 0;
    if (pDir->DeviceDescriptorPtr->bDescriptorType != USB_DEVICE_DESCRIPTOR_TYPE)
        return 0;
    if ((pDir->DeviceDescriptorPtr->bcdUSB < 0x100) && (pDir->DeviceDescriptorPtr->bcdUSB > 0x200))
        return 0;
    if (pDir->DeviceDescriptorPtr->bDeviceClass != 0) /*   */
        return 0;
    if (pDir->DeviceDescriptorPtr->bMaxPacketSize0 != 8)
        return 0;
    if (pDir->DeviceDescriptorPtr->idVendor != DEVICE_VID)
        return 0;
    if (pDir->DeviceDescriptorPtr->idProduct != DEVICE_PID)
        return 0;
    if (pDir->DeviceDescriptorPtr->iManufacturer > pDir->StringCount)
        return 0;
    if (pDir->DeviceDescriptorPtr->iProduct > pDir->StringCount)
        return 0;
    if (pDir->DeviceDescriptorPtr->iSerialNumber > pDir->StringCount)
        return 0;
    if (pDir->DeviceDescriptorPtr->bNumConfigurations != 1) /*   */
        return 0;
    /* bDeviceSubClass, bDeviceProtocol, bcdDevice   */

    /*   */
    if ((pDir->StrLangList[0] >> 8) != USB_STRING_DESCRIPTOR_TYPE)
        return 0;

    lang_count = (pDir->StrLangList[0] & 0xFF) / 2 - 1;
    if (lang_count < 0)
        return 0;

    for (i = 0; i < lang_count * pDir->StringCount; i += pDir->StringCount)
        {
        for (k = 0; k < pDir->StringCount; k++)
            {
            const ushort *pstr = pDir->String[i + k];
            if (!PTR_IN_FLASH(pstr))
                return 0;
            if ((pstr[0] >> 8) != USB_STRING_DESCRIPTOR_TYPE)
                return 0;
            }
        }

    /*   */
    if (pDir->ConfigDescriptorPtr->Conf.bLength != sizeof(USB_CONFIGURATION_DESCRIPTOR))
        return 0;
    if (pDir->ConfigDescriptorPtr->Conf.bDescriptorType != USB_CONFIGURATION_DESCRIPTOR_TYPE)
        return 0;
    if (pDir->ConfigDescriptorPtr->Conf.bNumInterfaces != 1) /*   */
        return 0;
    if (pDir->ConfigDescriptorPtr->Conf.bConfigurationValue != 1) /*    1 */
        return 0;
    if (pDir->ConfigDescriptorPtr->Conf.wTotalLength !=
        sizeof(USB_CONFIGURATION_DESCRIPTOR) + sizeof(USB_INTERFACE_DESCRIPTOR) +
        (sizeof(USB_ENDPOINT_DESCRIPTOR) * pDir->ConfigDescriptorPtr->Interface.bNumEndpoints))
        return 0;
    if (pDir->ConfigDescriptorPtr->Conf.iConfiguration > pDir->StringCount)
        return 0;
    /* bmAttributes, bMaxPower   */

    /*   */
    if (pDir->ConfigDescriptorPtr->Interface.bLength != sizeof(USB_INTERFACE_DESCRIPTOR))
        return 0;
    if (pDir->ConfigDescriptorPtr->Interface.bDescriptorType != USB_INTERFACE_DESCRIPTOR_TYPE)
        return 0;
    if (pDir->ConfigDescriptorPtr->Interface.bNumEndpoints > 3) /* SAM7 has 4 EP (ctl + 3) */
        return 0;
    if (pDir->ConfigDescriptorPtr->Interface.bInterfaceClass != 0) /*   */
        return 0;
    if (pDir->ConfigDescriptorPtr->Interface.iInterface > pDir->StringCount)
        return 0;
    /* bInterfaceNumber, bAlternateSetting, bInterfaceSubClass, bInterfaceProtocol   */

    /*  endpoint' */
    for (k = 0; k < pDir->ConfigDescriptorPtr->Interface.bNumEndpoints; k++)
        {
        if (pDir->ConfigDescriptorPtr->Endpoints[k].bLength != sizeof(USB_ENDPOINT_DESCRIPTOR))
            return 0;
        if (pDir->ConfigDescriptorPtr->Endpoints[k].bDescriptorType != USB_ENDPOINT_DESCRIPTOR_TYPE)
            return 0;
        if (pDir->ConfigDescriptorPtr->Endpoints[k].wMaxPacketSize > 64)
            return 0;
        /* bEndpointAddress, bmAttributes, bInterval   */
        }

    /* ,   */
    g_DescDirPtr = pDir;
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
