/*================================================================================================*
 *     USB (control channel) * *
 *================================================================================================*/

#include "USBControl.h"
#include "USBHardware.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void USBCTL_Read                    /*     */
    (
    unsigned int USBRequestSize,    /* ()  -   */
    void *pBuffer,                  /* ()  -    */
    unsigned int BufferSize         /* ()  -   (    USBRequestSize) */
    )
    {
    unsigned char* pData = (unsigned char*)pBuffer;
    unsigned char* pEnd = (unsigned char*)pBuffer + USBRequestSize;
    int pkt_size = 0;
    if (USBRequestSize > BufferSize)
        pEnd = (unsigned char*)pBuffer + BufferSize;
    while ((pData < pEnd) || (pkt_size == CONTROL_ENDPOINT_SIZE))
        {
        pkt_size = pEnd - pData;
        if (pkt_size > CONTROL_ENDPOINT_SIZE)
            pkt_size = CONTROL_ENDPOINT_SIZE;
        /*    FIFO */
        mem2reg(&AT91C_BASE_UDP->UDP_FDR[CONTROL_ENDPOINT], pData, pkt_size);
        pData += pkt_size;
        /*  ISOERROR  TXCOMP,    */
        Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_ISOERROR | AT91C_UDP_TXCOMP);
        /*   */
        Udp_ep_set_flag(CONTROL_ENDPOINT, AT91C_UDP_TXPKTRDY);
        /*     */
        while (!(AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] & AT91C_UDP_TXCOMP))
            { /*     ,    */
            if (AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] & (AT91C_UDP_RX_DATA_BK0 | AT91C_UDP_RXSETUP))
                {
                Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_RX_DATA_BK0);
                //Clear_Usb_Interrupts(1 << CONTROL_ENDPOINT);
                return;
                }
            }
        }
    /*   . */
    Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_TXCOMP);
    Clear_Usb_Interrupts(1 << CONTROL_ENDPOINT);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void USBCTL_Write                   /*     */
    (
    unsigned int USBRequestSize,    /* ()  -   */
    void *pBuffer,                  /* ()  -    */
    unsigned int BufferSize         /* ()  -   (    USBRequestSize) */
    )
    {
    unsigned char* pData = (unsigned char*)pBuffer;
    unsigned char* pEnd = (unsigned char*)pBuffer + USBRequestSize;
    int pkt_size;
    if (USBRequestSize > BufferSize)
        {
        USBCTL_RequestError();
        return;
        }
    while (pData < pEnd)
        {
        /*    */
        while (!(AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] & AT91C_UDP_RX_DATA_BK0))
            ;
        /*    */
        pkt_size = (AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] & AT91C_UDP_RXBYTECNT) >> 16;
        if (pkt_size > pEnd - pData)
            pkt_size = pEnd - pData;
        reg2mem(pData, &AT91C_BASE_UDP->UDP_FDR[CONTROL_ENDPOINT], pkt_size);
        pData += pkt_size;
        Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_RX_DATA_BK0);
        }
    /*    -    */
    USBCTL_RequestOK();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void USBCTL_RequestError            /*   ( stall) */
    (void)
    {
    /*  stall */
    Udp_ep_set_flag(CONTROL_ENDPOINT, AT91C_UDP_FORCESTALL);
    while (!(AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] & AT91C_UDP_ISOERROR))
        ;
    Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_FORCESTALL | AT91C_UDP_ISOERROR);
    Clear_Usb_Interrupts(1 << CONTROL_ENDPOINT);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void USBCTL_RequestOK               /*   (  ) */
    (void)
    {
    /*     */
    Udp_ep_set_flag(CONTROL_ENDPOINT, AT91C_UDP_TXPKTRDY);
    while (!(AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] & AT91C_UDP_TXCOMP))
        ;
    Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_TXCOMP);
    Clear_Usb_Interrupts(1 << CONTROL_ENDPOINT);
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
