#include "USBHardware.h"
#include "USBControl.h"
#include "main.h"
#if USB_FEATURE_BACKGROUND
#include "USBBackground.h"
#define FEATURE_RequestOK       USBBG_RequestOK
#define FEATURE_RequestError    USBBG_RequestError
#else
#define FEATURE_RequestOK       USBCTL_RequestOK
#define FEATURE_RequestError    USBCTL_RequestError
#endif


enum Standart_Request_Code
{
    GET_STATUS=0,
    CLEAR_FEATURE=1,
    Reserved=2,
    SET_FEATURE=3,
    Reserved1 =4,
    SET_ADDRESS=5,
    GET_DESCRIPTOR=6,
    SET_DESCRIPTOR=7,
    GET_CONFIGURATION=8,
    SET_CONFIGURATION=9,
    SET_INTERFACE,
    GET_INTERFACE,
    SYNCH_FRAME=12
};

enum Descriptor_Type
{
    DEVICE=1,
    CONFIGURATION,
    STRING,
    INTERFACE,
    ENDPOINT,
    DEVICE_QUALIFIER,
    OTHER_SPEED_CONFIGURATION,
    INTERFACE_POWER
};

/* Table 9-4. Standard Request Codes */
USB_request_Handler StandardDeviceRequest[] =
{
    std_getStatus,        /*  0 */
    std_clearFeature,     /*  1 */
    std_reserved,         /*  2 */
    std_setFeature,       /*  3 */
    std_reserved,         /*  4 */
    std_setAddress,       /*  5 */
    std_getDescriptor,    /*  6 */
    std_setDescriptor,    /*  7 */
    std_getConfiguration, /*  8 */
    std_setConfiguration, /*  9 */
    std_getInterface,     /* 10 */
    std_setInterface,     /* 11 */
    std_synch_frame       /* 12 */
};


USBRequest CurrentUSBRequest;

/*
  ,      
*/
void ControlCommandProcessing()
    {
    register int i;

    for (i = 0; i < 8; i++)
        {
        ((byte *)&CurrentUSBRequest)[i] = AT91C_BASE_UDP->UDP_FDR[CONTROL_ENDPOINT];
        }

    if ((CurrentUSBRequest.bmRequestType & REQUEST_ANSWER_DIRECTION) == REQUEST_ANSWER_DIRECTION_DEVICE_TO_HOST)
        { /*    USB   */
        Udp_ep_set_flag(CONTROL_ENDPOINT, AT91C_UDP_DIR);
        }
    else
        { /*      USB */
        Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_DIR);
        }
    /*  RXSETUP */
    Udp_ep_clr_flag(CONTROL_ENDPOINT, AT91C_UDP_RXSETUP);

    CurrentUSBRequest.bRequest &= FW_USB_REQUEST_MASK; //    4 
    switch (CurrentUSBRequest.bmRequestType & REQUEST_TYPE)
        {
        case REQUEST_TYPE_STANDARD: //  USB Request
            if (CurrentUSBRequest.bRequest < sizeof(StandardDeviceRequest)/sizeof(StandardDeviceRequest[0]))
                {
                StandardDeviceRequest[CurrentUSBRequest.bRequest]();
                }
            else
                {
                USBCTL_RequestError();
                }
            break;
        case REQUEST_TYPE_VENDOR: //  USB Request
            if (CurrentUSBRequest.bRequest < VDRQ_COUNT)
                {
                VendorDeviceRequest[CurrentUSBRequest.bRequest]();
                }
            else
                {
                USBCTL_RequestError();
                }
            break;
        case REQUEST_TYPE_CLASS:
            break;
        default:
            break;
        }
    }

struct _StatusStruct
    {
    ushort ZeroStatus, DeviceStatus, InterfaceStatus;
    byte Configuration;
    ushort EndPointsStalled[4];
    }
    StatusStruct;

void std_reserved(void)
    {
    USBCTL_RequestError();
    }

void std_getStatus(void)
    {
    int index = 0;
    switch (CurrentUSBRequest.bmRequestType)
        {
        case DEVICE_IN:  // device status
            // A GetStatus() request to a device returns the information shown in Figure 9-4.
            // DO=0 : Bus powered
            StatusStruct.DeviceStatus = 0;
            USBCTL_Read(CurrentUSBRequest.wLength,
                &StatusStruct.DeviceStatus, sizeof(StatusStruct.DeviceStatus));
            break;
        case INTERFACE_IN: // interface status
            USBCTL_Read(CurrentUSBRequest.wLength,
                &StatusStruct.InterfaceStatus, sizeof(StatusStruct.InterfaceStatus));
            break;
        case ENDPOINT_IN: // endpoint status
            index = CurrentUSBRequest.wIndex & 0x0F;
            if (index < 4)
                {
                USBCTL_Read(CurrentUSBRequest.wLength,
                    &StatusStruct.EndPointsStalled[index],
                    sizeof(StatusStruct.EndPointsStalled[index]));
                }
            else
                {
                USBCTL_RequestError();
                }
            break;
        default:
            USBCTL_RequestError();
            break;
        }
    }

void std_clearFeature(void)
    {
    int index = 0;
    switch (CurrentUSBRequest.bmRequestType)
        {
        case DEVICE_OUT:   // device status
            FEATURE_RequestError();
            break;
        case INTERFACE_OUT: // interface status
            FEATURE_RequestOK();
            break;
        case ENDPOINT_OUT:  // endpoint status
            index = CurrentUSBRequest.wIndex & 0x0F;
            if ((CurrentUSBRequest.wValue == FEATURE_ENDPOINT_HALT) && (index < 4))
                {
                SetEndpointStall(index, false);
                StatusStruct.EndPointsStalled[index] = 0;
                /* Reset endpoint */
                ResetBulkEndpoint(index);
                FEATURE_RequestOK();
                }
            else
                {
                FEATURE_RequestError();
                }
            break;
        default:
            FEATURE_RequestError();
            break;
        }
    }

void std_setFeature(void)
    {
    int index = 0;
    switch (CurrentUSBRequest.bmRequestType)
        {
        case (DEVICE_OUT):  // device status
            FEATURE_RequestOK();
            break;
        case INTERFACE_OUT: // interface status
            FEATURE_RequestOK();
            break;
        case ENDPOINT_OUT: // endpoint status
            index = CurrentUSBRequest.wIndex & 0x0F;
            if ((CurrentUSBRequest.wValue == FEATURE_ENDPOINT_HALT) && (index < 4))
                {
                SetEndpointStall(index, true);
                StatusStruct.EndPointsStalled[index] = 1;
                FEATURE_RequestOK();
                }
            else
                {
                FEATURE_RequestError();
                }
            break;
        default:
            FEATURE_RequestError();
            break;
        }
    }

void std_setAddress(void)
    {
    if (CurrentUSBRequest.bmRequestType == DEVICE_OUT)
        {
        USBCTL_RequestOK();
        USBSetAddress(CurrentUSBRequest.wValue & FW_DEVICE_ADDRESS_MASK);
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void std_getDescriptor(void)
    {
    int DescriptorType, DescriptorIndex;

    if (CurrentUSBRequest.bmRequestType != DEVICE_IN)
        {
        USBCTL_RequestError();
        return;
        }

    DescriptorType = CurrentUSBRequest.wValue >> 8;
    DescriptorIndex = CurrentUSBRequest.wValue & 0xFF;

    switch (DescriptorType)
        {
        case DEVICE:
            USBCTL_Read(CurrentUSBRequest.wLength,
                (void*)g_DescDirPtr->DeviceDescriptorPtr,
                g_DescDirPtr->DeviceDescriptorPtr->bLength);
            break;
        case INTERFACE:
        case ENDPOINT:
        case CONFIGURATION:
            USBCTL_Read(CurrentUSBRequest.wLength,
                (void*)g_DescDirPtr->ConfigDescriptorPtr,
                sizeof(FULL_CONFIG_DESCRIPTOR));
            break;
        case STRING:
            if (DescriptorIndex == 0) /* language list */
                {
                USBCTL_Read(CurrentUSBRequest.wLength,
                    (void*)g_DescDirPtr->StrLangList,
                    g_DescDirPtr->StrLangList[0] & 0xFF /* bLength */ );
                }
            else if (DescriptorIndex <= g_DescDirPtr->StringCount)
                {
                int lang_count, i, lang;
                const ushort *pData;
                /*    */
                lang_count = ((g_DescDirPtr->StrLangList[0] & 0xFF) / 2) - 1;
                /*   ( ,     ) */
                lang = 0;
                for (i = 0; i < lang_count; i++)
                    {
                    if (CurrentUSBRequest.wIndex == g_DescDirPtr->StrLangList[1 + i])
                        {
                        lang = i;
                        break;
                        }
                    }
                pData = g_DescDirPtr->String[lang * g_DescDirPtr->StringCount + DescriptorIndex - 1];
                USBCTL_Read(CurrentUSBRequest.wLength,
                    (void*)pData, pData[0] & 0xFF /* bLength */);
                }
            else
                {
                USBCTL_RequestError();
                }
            break;
        case DEVICE_QUALIFIER:
            USBCTL_RequestError();
            break; //   high speed
        case OTHER_SPEED_CONFIGURATION:
            USBCTL_RequestError();
            break;
        case INTERFACE_POWER:
            USBCTL_RequestError();
            break;
        default:
            USBCTL_RequestError();
            break;
        }
    }

void std_setDescriptor(void)
    {
    if (CurrentUSBRequest.bmRequestType == DEVICE_OUT)
        {
        USBCTL_RequestOK();
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void std_getConfiguration(void)
    {
    if (CurrentUSBRequest.bmRequestType == DEVICE_IN)
        {
        USBCTL_Read(CurrentUSBRequest.wLength,
            &StatusStruct.Configuration,
            sizeof(StatusStruct.Configuration));
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void std_setConfiguration(void)
    {
    if (CurrentUSBRequest.bmRequestType == DEVICE_OUT)
        {
        USBCTL_RequestOK();

        // Disable_USB_Interrupts(DISABLE_ALL_IT);
        // Clear_Usb_Interrupts(CLEAR_ALL_IT);

        StatusStruct.DeviceStatus = 1; // SELF_POWERED
        StatusStruct.InterfaceStatus = 0;
        StatusStruct.Configuration = CurrentUSBRequest.wValue;
        StatusStruct.EndPointsStalled[0] = 0;
        StatusStruct.EndPointsStalled[1] = 0;
        StatusStruct.EndPointsStalled[2] = 0;
        StatusStruct.EndPointsStalled[3] = 0;

        if (CurrentUSBRequest.wValue == 1)
            {
            SetGlobalStateConfig();
            ConfigureEndpoint(IN_ENDPOINT, BULK_IN, true);
            ConfigureEndpoint(OUT_ENDPOINT, BULK_OUT, true);
            Enable_USB_Interrupts(AT91C_UDP_EPINT1 | AT91C_UDP_EPINT2);
            USBConfigured = true;
            }
        else
            {
            ClrGlobalStateConfig();
            DisableEndpoint(IN_ENDPOINT);
            DisableEndpoint(OUT_ENDPOINT);
            }
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void std_getInterface()
    {
    if (CurrentUSBRequest.bmRequestType == INTERFACE_IN)
        {
        USBCTL_RequestError();
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void std_setInterface()
    {
    if (CurrentUSBRequest.bmRequestType == INTERFACE_OUT)
        {
        USBCTL_RequestError();
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void std_synch_frame()
    {
    if (CurrentUSBRequest.bmRequestType == 0x82)
        {
        USBCTL_RequestError();
        }
    else
        {
        USBCTL_RequestError();
        }
    }

void EndBusResProcessing()
    {
    ResetUSB();

    ConfigureEndpoint(CONTROL_ENDPOINT, CONTROL, true);
    ConfigureEndpoint(IN_ENDPOINT, BULK_IN, false);
    ConfigureEndpoint(OUT_ENDPOINT, BULK_OUT, false);

    Disable_USB_Interrupts(DISABLE_ALL_IT);
    Clear_Usb_Interrupts(CLEAR_ALL_IT);

    /* USB Interrupt Enable Register */
    /* AT91C_UDP_EPINT0 : Endpoint 0 Interrupt          */
    /* AT91C_UDP_RXSUSP : USB Suspend Interrupt         */
    /* AT91C_UDP_RXRSM  : USB Resume Interrupt          */
    /* AT91C_UDP_EXTRSM : USB External Resume Interrupt */
    /* AT91C_UDP_SOFINT : USB Start Of frame Interrupt  */
    Enable_USB_Interrupts(  AT91C_UDP_EPINT0
                            | AT91C_UDP_RXSUSP
                            | AT91C_UDP_RXRSM  | AT91C_UDP_EXTRSM
//                         | AT91C_UDP_SOFINT
                            | AT91C_UDP_WAKEUP );
    }

void ExitSleepMode()
    {
    SetDeviceClock(USB_PORT_CLOCK, true); //  USB
    Disable_USB_Interrupts(DISABLE_ALL_IT); //   
    Clear_Usb_Interrupts(CLEAR_ALL_IT); //   
    Enable_USB_Interrupts(  AT91C_UDP_EPINT0 | AT91C_UDP_EPINT1
                            | AT91C_UDP_EPINT2 | AT91C_UDP_RXSUSP
                            | AT91C_UDP_RXRSM  | AT91C_UDP_EXTRSM
//                          | AT91C_UDP_SOFINT
                            | AT91C_UDP_WAKEUP );
    AT91C_BASE_UDP->UDP_TXVC &= ~AT91C_UDP_TXVDIS;
    USBSuspend = false;
    g_MainFlag |= MAIN_EXIT_SLEEP_MODE;
    }

void EnterSleepMode()
    {
    Disable_USB_Interrupts(DISABLE_ALL_IT); //   
    Clear_Usb_Interrupts(CLEAR_ALL_IT); //   
    Enable_USB_Interrupts(AT91C_UDP_RXRSM);
    AT91C_BASE_UDP->UDP_TXVC |= AT91C_UDP_TXVDIS;
    SetDeviceClock(USB_PORT_CLOCK, false); //  USB
    USBSuspend = true;
    g_MainFlag |= MAIN_ENTER_SLEEP_MODE;
    }

