#include "HAL.h"
#include "clk.h"
#include "timers.h"
#include "debug.h"
#include "wdt.h"
#include "pio.h"

/*      */
extern void Spurious_Handler_Entry(void);
extern void FIQ_Handler_Entry(void);
extern void IRQ_Handler_Entry(void);

/*------------------------------------------------------------------------------------------------*/
void InitializeInterrupts()
    {
    DisableInterruptsMask(0xFFFFFFFF);
    /*         */
    AT91F_AIC_Open(
        AT91C_BASE_AIC,             /* pointer to the AIC registers */
        IRQ_Handler_Entry,          /* Default IRQ vector exception */
        FIQ_Handler_Entry,          /* Default FIQ vector exception */
        IRQ_Handler_Entry,          /* Default Handler set in ISR */
        Spurious_Handler_Entry,     /* Default Spurious Handler */
        0x0);                       /* Debug Control Register */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*
 *   ,    Cstartup.s79
 *------------------------------------------------------------------------------------------------*/
void InitializeHardware()
    {
    __disable_interrupt();

#if 0
    WDT_Disable();
#else
    /*    watchdog */
    WDT_Init();     /* ,    bootrom' */
    WDT_Restart();  /* ,    */
    /* --> watchdog    */
#endif

    CLK_InitClocks();
    /*     ( ,    Init) */
    EnablePeripheralClock((1<<AT91C_ID_SYS)|(1<<AT91C_ID_PIOA)|(1<<AT91C_ID_UDP));

    PIO_Init();

    InitializeInterrupts();

    /*  User Reset    1 ms */
    AT91C_BASE_RSTC->RSTC_RMR = 0xA5000000 |
        (AT91C_RSTC_ERSTL & (4 << 8)) |
        AT91C_RSTC_URSTEN;

#if DEBUG_OUTPUT_MCK_PCK1
    /*    PCK1 */
    AT91C_BASE_PMC->PMC_PCKR[1] = AT91C_BASE_PMC->PMC_MCKR;
    AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_PCK1;
    AT91C_BASE_PIOA->PIO_BSR = AT91C_PA21_PCK1;
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA21_PCK1;
#endif

    }
/*------------------------------------------------------------------------------------------------*/
