/*================================================================================================*
 *   E-140
 *================================================================================================*/

#include "adc.h"
#include "buffers.h"
#include "cpld.h"
#include "chipio.h"
#include "main.h"
#include "USBHardware.h"
#include "debug.h"
#include "pio.h"

/*================================================================================================*/
/*   */
#define ADC_ST_STOPPED      0       /*   */
#define ADC_ST_RUNNING      1       /*     ( DMA) */
#define ADC_ST_FRAME_END    2       /*     */
#define ADC_ST_FRAME_OUT    3       /*      USB */

#define ADC_FREQDIV_200KHZ  (8000/200 - 1)

/*         ,   */
#define CHAN_SWAP(ch) ((((ch) & 0x03) << 2) | (((ch) & 0x0C) >> 2) | ((ch) & 0xF0))

/*      N */
#define RING_NEXT(idx, N)  (((idx) < ((N) - 1)) ? ((idx) + 1) : 0)

/*   ()   */
#define BS_EMPTY            0       /*  */
#define BS_READY            1       /*   ,    */
#define BS_FILLING          2       /*       */
#define BS_OVERRUN          3       /*         */

#define USB_IN_FREE()       (USB_IN_Array_Size - USB_IN_ArrayCounter)
/*================================================================================================*/

/*================================================================================================*/
/*   ( ,   ) */
static t_ADC_ParamStruct f_ADC_Params;

int g_ADC_StopReceived = 0;

/*       */
short int g_ADC_SingleResult = 0;

/*     */
static unsigned int f_ReadIdx = 0;
static volatile unsigned int f_WriteIdx = 0;
static volatile unsigned char f_BufState[ADC_NBLOCKS];
static unsigned int f_FrameSize = 0;

/*   9-     (    SSC) */
static unsigned short int f_ADC_ControlTab[ADC_MAX_CH];
static unsigned int f_ControlTabLen; /*   f_ADC_ControlTab */

/*        */
static volatile int f_RunState = ADC_ST_STOPPED;
/*    */
static int f_SingleFrame = 0;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
__arm __ramfunc void SSC_Interrupt_Handler
    (void)
    {
    unsigned int status;

    status = AT91C_BASE_SSC->SSC_SR;
    status &= AT91C_BASE_SSC->SSC_IMR;

    if (f_RunState != ADC_ST_RUNNING)
        {
        AT91C_BASE_SSC->SSC_IDR = 0xFFFFFFFF;
        return;
        }

#if DEBUG_LED_ADC_INTERRUPT
    DEBUG_LED_ON();
#endif

    /*      (  TNPR/TNCR) */
    if (status & AT91C_SSC_ENDTX)
        {
        AT91C_BASE_SSC->SSC_TNPR = (unsigned int)f_ADC_ControlTab;
        AT91C_BASE_SSC->SSC_TNCR = f_ControlTabLen;
        }

    if (status & AT91C_SSC_ENDRX)
        {
        if (!f_SingleFrame)
            { /*    */
            unsigned int next;

            /*    ,  DMA    
             *   f_WriteIdx        ( 
             *   RNPR/RNCR).
             *
             *      BS_FILLING  BS_OVERRUN.
             *   BS_OVERRUN,   ,        
             * "",  DMA      .
             *     f_WriteIdx  .
             *      ( BS_FILLING),    
             *   (BS_READY),   f_WriteIdx   .
             */
            if (f_BufState[f_WriteIdx] == BS_FILLING)
                {
                f_BufState[f_WriteIdx] = BS_READY;
                f_WriteIdx = RING_NEXT(f_WriteIdx, ADC_NBLOCKS);
                g_Diag.ADC_SampleCount += ADC_BLK_SIZE_W;
                }
            /* ,       */
            next = RING_NEXT(f_WriteIdx, ADC_NBLOCKS);
            if (f_BufState[next] == BS_EMPTY)
                {
                /*   =>    BS_FILLING, DMA    */
                f_BufState[f_WriteIdx] = BS_FILLING;
                AT91C_BASE_SSC->SSC_RNPR = (unsigned int)g_Buf.ADC.asRing[next];
                }
            else
                {
                /*   =>    BS_OVERRUN, DMA     */
                f_BufState[f_WriteIdx] = BS_OVERRUN;
                INC_SATURATE(g_Diag.ADC_OverrunCount);
                AT91C_BASE_SSC->SSC_RNPR = (unsigned int)g_Buf.ADC.asRing[f_WriteIdx];
                }
            AT91C_BASE_SSC->SSC_RNCR = ADC_BLK_SIZE_W;
            }
        else /* f_SingleFrame */
            { /*     */
            f_RunState = ADC_ST_FRAME_END;
            AT91C_BASE_SSC->SSC_IDR = AT91C_SSC_ENDRX; /*    */
            }
        }
#if DEBUG_LED_ADC_INTERRUPT
    DEBUG_LED_OFF();
#endif
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_ClearRingBuf          /*     */
    (void)
    {
    /*       */
    unsigned int k;

    /*    */
    __disable_interrupt();
    f_ReadIdx = 0;
    f_WriteIdx = 0;
    f_FrameSize = 0;
    /*      , ..       */
    f_BufState[0] = BS_FILLING;
    for (k = 1; k < ADC_NBLOCKS; k++)
        {
        f_BufState[k] = BS_EMPTY;
        }
    __enable_interrupt();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static int f_StartADC               /*    */
    (
    unsigned int Mode               /*   (CPLD_ADCSTART_...) */
    )                               /*  1  , 0   */
    /*       ADC_ST_STOPPED */
    {
    f_RunState = ADC_ST_RUNNING;
    /*  PDC  SSC   */
    AT91C_BASE_SSC->SSC_IER = AT91C_SSC_ENDRX;
    AT91C_BASE_SSC->SSC_PTCR = AT91C_PDC_RXTEN;
    AT91C_BASE_SSC->SSC_CR = AT91C_SSC_RXEN;
    /*     (    ) */
    if (!CPLD_ADC_Start(Mode))
        {
        AT91C_BASE_SSC->SSC_IDR = AT91C_SSC_ENDRX;
        AT91C_BASE_SSC->SSC_PTCR = AT91C_PDC_RXTDIS;
        AT91C_BASE_SSC->SSC_CR = AT91C_SSC_RXDIS;
        f_RunState = ADC_ST_STOPPED;
        return 0;
        }
#if DEBUG_LED_ADC_RUNNING
    DEBUG_LED_ON();
#endif
    DIAG_SET_FLAG(DIAG_DEVSTATE_ADC_STARTED);
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
static void f_StopSSC               /*  SSC   DMA */
    (void)
    {
    /*    */
    AT91C_BASE_SSC->SSC_IDR = 0xFFFFFFFF;
    /*  DMA  SSC */
    AT91C_BASE_SSC->SSC_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;

    /*      */
    AT91C_BASE_SSC->SSC_CR = AT91C_SSC_SWRST;
    AT91C_BASE_SSC->SSC_CR = AT91C_SSC_TXDIS | AT91C_SSC_RXDIS;

    /*  SSC:
     * : 16 ,   RF falling, 1 /, MSB first.
     * : 9 ,   TF falling, 1 /, MSB first.
     */
    AT91C_BASE_SSC->SSC_CMR = 0; /*  clock divider (  clock) */
    /*   */
    AT91C_BASE_SSC->SSC_RCMR =
        (AT91C_SSC_CKS & 0x02) |        /*  rxclk = RK   */
        AT91C_SSC_CKO_NONE |
        (AT91C_SSC_CKI | 0) |           /*  rxclk (1 =    ) */
        AT91C_SSC_START_FALL_RF |       /*   RF falling edge */
        (AT91C_SSC_STTDLY & (0 << 16)) |
        (AT91C_SSC_PERIOD & (0 << 24));
    AT91C_BASE_SSC->SSC_RFMR =
        (AT91C_SSC_DATLEN & 15) |       /* 16  */
        (AT91C_SSC_LOOP & 0) |          /* no loopback */
        AT91C_SSC_MSBF |                /* MSB first */
        (AT91C_SSC_DATNB & (0 << 8)) |  /* DATNB+1 = 1 words per frame */
        (AT91C_SSC_FSLEN & (0 << 16)) | /* No frame sync bits */
        AT91C_SSC_FSOS_NONE |           /* No frame sync (RF = input) */
        AT91C_SSC_FSEDGE;               /* RXSYN interrupt on falling edge of framesync (unused?) */
    /*   */
    /* -  CKI = 1  STTDLY = 0,  CKI  .
        CKI = 0  STTDLY = 0,    . */
    AT91C_BASE_SSC->SSC_TCMR =
        (AT91C_SSC_CKS & 0x02) |        /*  txclk = TK   */
        AT91C_SSC_CKO_NONE |
        (AT91C_SSC_CKI | 0) |           /*  txclk (TD    ) */
        AT91C_SSC_START_FALL_RF |       /*   TF falling edge */
        (AT91C_SSC_STTDLY & (0 << 16)) | /* Start delay, clk */
        (AT91C_SSC_PERIOD & (0 << 24));
    AT91C_BASE_SSC->SSC_TFMR =
        (AT91C_SSC_DATLEN & 8) |        /* 9  */
        (AT91C_SSC_DATDEF & 0) |        /* TD level when not transmitting */
        AT91C_SSC_MSBF |                /* MSB first */
        (AT91C_SSC_DATNB & (0 << 8)) |  /* DATNB+1 = 1 words per frame */
        (AT91C_SSC_FSLEN & (0 << 16)) | /* No frame sync bits */
        AT91C_SSC_FSOS_NONE |           /* No frame sync (TF = input) */
        (AT91C_SSC_FSDEN & 0) |         /* No transmit sync data */
        AT91C_SSC_FSEDGE;               /* TXSYN interrupt on falling edge of framesync (unused?) */

    /*      ,    RXRDY */
    if (AT91C_BASE_SSC->SSC_RHR);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ADC_Init                       /*      SSC */
    (void)
    {
    f_RunState = ADC_ST_STOPPED;
    DIAG_CLEAR_FLAG(DIAG_DEVSTATE_ADC_STARTED);
    g_Diag.ADC_OverrunCount = 0;
    g_Diag.ADC_SampleCount = 0;

    /*    */
    f_ADC_Params.ChanCtl[0] = 0;
    f_ADC_Params.SampleRateDiv = 80; /* 100  */
    f_ADC_Params.NumChannelsUsed = 1;
    f_ADC_Params.InterFrameDelay = 0;
    f_ADC_Params.SyncMode = ADC_SYNC_SOFT;
    f_ADC_Params.SyncChanCtl = 0;
    f_ADC_Params.SyncThreshold = 0;

    /*   SSC */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_SSC);

    /*   */
    f_StopSSC();

    /*    */
    f_ClearRingBuf();

    /*  SSC    */
    AT91C_BASE_PIOA->PIO_ASR =
        AT91C_PA15_TF | AT91C_PA16_TK | AT91C_PA17_TD |
        AT91C_PA20_RF | AT91C_PA19_RK | AT91C_PA18_RD;
    AT91C_BASE_PIOA->PIO_PDR =
        AT91C_PA15_TF | AT91C_PA16_TK | AT91C_PA17_TD |
        AT91C_PA20_RF | AT91C_PA19_RK | AT91C_PA18_RD;

    /*    */
    SetInterrupt(AT91C_ID_SSC, SSC_Interrupt_Handler, PRI_HIGH, INTERNAL_HIGH_LEVEL);
    EnableInterruptsMask(1 << AT91C_ID_SSC);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int ADC_IsRunning                   /* ,     (  ) */
    (void)
    {
    return (f_RunState != ADC_ST_STOPPED);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ADC_GetParams                  /*    ( ) */
    (
    t_ADC_ParamStruct *pParams
    )
    {
    if (pParams)
        memcpy(pParams, &f_ADC_Params, sizeof(f_ADC_Params));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ADC_SetParams                  /*    ( ) */
    (
    const t_ADC_ParamStruct *pParams
    )
    {
    if (pParams)
        memcpy(&f_ADC_Params, pParams, sizeof(f_ADC_Params));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int ADC_Start                       /*      */
    (
    int SingleFrame                 /* ()  - 0 =  , 1 =   */
    )                               /*  1  , 0   */
/*  :
 *     SSC    DMA. SSC .
 *     .
 *           
 *     f_ADC_Params,        .
 */
    {
    int k;
    unsigned int adc_mode, start_mode, frame_ofs;
    unsigned int NumChanUsed; /*    ( ) */

    if (f_RunState != ADC_ST_STOPPED)
        return 0;
    if (!CPLD_PowerGood())
        return 0;
    if ((f_ADC_Params.NumChannelsUsed == 0) || (f_ADC_Params.NumChannelsUsed > ADC_MAX_CH))
        return 0;

    /*  :
     * ---------------------------------------------------------------------
     *                f_ADC_Params         
     *       8000000 / Fadc              (8000000 / Fadc) - 1
     *                      min = 40                    min = 39
     *                      max = 0xFFFF                max = 0xFFFE
     *   (T * Fadc) - 1              (T * Fadc) - 1
     *                      min = 0                     min = 0
     *                      max = 255                   max = 255
     * ---------------------------------------------------------------------
     */
    if (f_ADC_Params.SampleRateDiv <= ADC_FREQDIV_200KHZ)
        return 0;

    /*  SSC  PDC */
    f_StopSSC();

    /*    */
    f_ClearRingBuf();

    /*    */
    NumChanUsed = f_ADC_Params.NumChannelsUsed;
    /*   ,     ,    
        DMA  SSC */
    f_ControlTabLen = (ADC_MAX_CH / NumChanUsed) * NumChanUsed;
    for (k = 0; k < NumChanUsed; k++)
        {
        unsigned int chan, cp;
        chan = f_ADC_Params.ChanCtl[k];
        chan = CHAN_SWAP(chan);
        /*     "1"    */
        if (k == NumChanUsed - 1)
            chan |= 0x100;
        for (cp = 0; cp < f_ControlTabLen; cp += NumChanUsed)
            {
            f_ADC_ControlTab[cp + k] = chan;
            }
        }

    adc_mode = 0;
    /*     */
    if (f_ADC_Params.SyncMode & ADC_SYNC_EXT_CLOCK)
        {
        adc_mode |= CPLD_ADCMODE_SAMPLESYNC_SLAVE_RISE;
        }
    else if (f_ADC_Params.SyncMode & ADC_SYNC_ENABLE_OUT)
        {
        adc_mode |= CPLD_ADCMODE_SAMPLESYNC_MASTER;
        }
    else
        {
        adc_mode |= CPLD_ADCMODE_SAMPLESYNC_INTERNAL;
        }

    /*      */
    if (SingleFrame)
        { /*         */
        adc_mode |= CPLD_ADCMODE_STARTSYNC_SOFT;
        f_SingleFrame = 1;
        start_mode = CPLD_ADCSTART_SINGLE_FRAME;
        /*        endpoint' */
        frame_ofs = (-(NumChanUsed * sizeof(short int))) % IN_ENDPOINT_SIZE;
        if (frame_ofs > 0)
            memset((char*)g_Buf.ADC.asFrame, 0, frame_ofs);
        /*     */
        f_FrameSize = frame_ofs + (NumChanUsed * sizeof(short int));
        }
    else
        { /*     */
        f_SingleFrame = 0;
        g_Diag.ADC_OverrunCount = 0; /*     */
        g_Diag.ADC_SampleCount = 0; /*     */
        start_mode = CPLD_ADCSTART_CONTINUOUS;
        switch (f_ADC_Params.SyncMode & ADC_SYNC_MODE_MASK)
            {
            case ADC_SYNC_SOFT:      /*   */
                adc_mode |= CPLD_ADCMODE_STARTSYNC_SOFT;
                break;
            case ADC_SYNC_EXT_START: /*    */
                adc_mode |= CPLD_ADCMODE_STARTSYNC_EXT_FALL;
                break;
            case ADC_SYNC_EXT_FRAME: /*    */
                adc_mode |= CPLD_ADCMODE_STARTSYNC_EXT_FALL;
                start_mode = CPLD_ADCSTART_FRAMESYNC;
                break;
            case ADC_SYNC_LVL_ABOVE: /*     ,  */
                adc_mode |= (CPLD_ADCMODE_STARTSYNC_ANALOG | CPLD_ADCMODE_ANALOG_LVL_ABOVE);
                break;
            case ADC_SYNC_LVL_BELOW: /*       ,  */
                adc_mode |= (CPLD_ADCMODE_STARTSYNC_ANALOG | CPLD_ADCMODE_ANALOG_LVL_BELOW);
                break;
            case ADC_SYNC_EDGE_RISE: /*      ,  */
                adc_mode |= (CPLD_ADCMODE_STARTSYNC_ANALOG | CPLD_ADCMODE_ANALOG_EDGE_RISE);
                break;
            case ADC_SYNC_EDGE_FALL: /*      ,  */
                adc_mode |= (CPLD_ADCMODE_STARTSYNC_ANALOG | CPLD_ADCMODE_ANALOG_EDGE_FALL);
                break;
            default:                 /* :   =   */
                adc_mode |= CPLD_ADCMODE_STARTSYNC_SOFT;
                break;
            }
        }

    /*    ,       */
    if ((adc_mode & CPLD_ADCMODE_STARTSYNC_MASK) == CPLD_ADCMODE_STARTSYNC_ANALOG)
        {
        unsigned int chan = f_ADC_Params.SyncChanCtl;
        if (!CPLD_ADC_SetupAnalogSync(CHAN_SWAP(chan), f_ADC_Params.SyncThreshold))
            return 0;
        }

    /*    USB */
    ResetBulkEndpoint(IN_ENDPOINT);

    /*   DMA (PDC)   f_ADC_ControlTab     */
    AT91C_BASE_SSC->SSC_TPR = (unsigned int)f_ADC_ControlTab;
    AT91C_BASE_SSC->SSC_TCR = f_ControlTabLen;
    AT91C_BASE_SSC->SSC_TNPR = (unsigned int)f_ADC_ControlTab;
    AT91C_BASE_SSC->SSC_TNCR = f_ControlTabLen;
    if (!f_SingleFrame)
        { /*    */
        AT91C_BASE_SSC->SSC_RPR = (unsigned int)g_Buf.ADC.asRing[0];
        AT91C_BASE_SSC->SSC_RNPR = (unsigned int)g_Buf.ADC.asRing[1];
        AT91C_BASE_SSC->SSC_RCR = ADC_BLK_SIZE_W;
        AT91C_BASE_SSC->SSC_RNCR = ADC_BLK_SIZE_W;
        }
    else
        { /*       */
        AT91C_BASE_SSC->SSC_RPR = ((unsigned int)g_Buf.ADC.asFrame) + frame_ofs;
        AT91C_BASE_SSC->SSC_RNPR = 0;
        AT91C_BASE_SSC->SSC_RCR = NumChanUsed;
        AT91C_BASE_SSC->SSC_RNCR = 0;
        }

    /*  PDC  SSC   */
    AT91C_BASE_SSC->SSC_IER = AT91C_SSC_ENDTX;
    AT91C_BASE_SSC->SSC_PTCR = AT91C_PDC_TXTEN;
    AT91C_BASE_SSC->SSC_CR = AT91C_SSC_TXEN;

    if (
        /*         SSC */
        CPLD_ADC_InitAndPreload(adc_mode)
        /*    */
        && CPLD_ADC_SetSamplingRate(f_ADC_Params.SampleRateDiv - 1)
        /*    */
        && CPLD_ADC_SetFrameDelay(f_ADC_Params.InterFrameDelay)
        && f_StartADC(start_mode)
        )
        {
        return 1;
        }

    /*  --  SSC  PDC */
    f_StopSSC();
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int ADC_StartSingleChannel          /*      g_ADC_SingleResult */
    (
    unsigned char ChanCtl           /* ()  -   */
    )                               /*  1  , 0   */
/*  :
 *           
 *     ,        .
 */
    {
    unsigned int chan;
    int k;

    if (f_RunState != ADC_ST_STOPPED)
        return 0;
    if (!CPLD_PowerGood())
        return 0;

    /*  SSC  PDC */
    f_StopSSC();

    /*     ,  ChanCtl */
    f_ControlTabLen = 4; /* 2  , 1 , +1    */
    chan = ChanCtl;
    chan = CHAN_SWAP(chan) | 0x100;
    for (k = 0; k < f_ControlTabLen; k++)
        {
        f_ADC_ControlTab[k] = chan;
        }

    f_SingleFrame = 1;
    f_FrameSize = 0; /*      USB */
    g_ADC_SingleResult = 0; /*  ,    */

    /*   DMA (PDC)   f_ADC_ControlTab   g_ADC_SingleResult */
    AT91C_BASE_SSC->SSC_TPR = (unsigned int)f_ADC_ControlTab;
    AT91C_BASE_SSC->SSC_TCR = f_ControlTabLen;
    AT91C_BASE_SSC->SSC_TNPR = (unsigned int)f_ADC_ControlTab;
    AT91C_BASE_SSC->SSC_TNCR = f_ControlTabLen;
    AT91C_BASE_SSC->SSC_RPR = (unsigned int)&g_ADC_SingleResult;
    AT91C_BASE_SSC->SSC_RNPR = 0;
    AT91C_BASE_SSC->SSC_RCR = 1;
    AT91C_BASE_SSC->SSC_RNCR = 0;

    /*  PDC  SSC   */
    AT91C_BASE_SSC->SSC_IER = AT91C_SSC_ENDTX;
    AT91C_BASE_SSC->SSC_PTCR = AT91C_PDC_TXTEN;
    AT91C_BASE_SSC->SSC_CR = AT91C_SSC_TXEN;

    if (
        /*         SSC */
        CPLD_ADC_InitAndPreload(CPLD_ADCMODE_SAMPLESYNC_INTERNAL | CPLD_ADCMODE_STARTSYNC_SOFT)
        && f_StartADC(CPLD_ADCSTART_SINGLE_FRAME)
        )
        {
        return 1;
        }

    /*  --  SSC  PDC */
    f_StopSSC();
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int ADC_Stop                        /*   ,    */
    (void)                          /*  1  , 0   */
    {
    if (f_RunState == ADC_ST_STOPPED)
        {
        g_ADC_StopReceived = 0;
        return 1; /*   */
        }

    /*  SSC  PDC   */
    f_StopSSC();

    /*    */
    f_ClearRingBuf();

    /*  endpoint (     ) */
    /* UPDATE 31-JUL-2009: -        USB
     *    ADC_STOP       ( CancelIo).
     *  ResetBulkEndpoint()    USBStack.c ("clear feature"),
     *   ,      (  
     * /).
     */
    //KNOWN TO CAUSE PROBLEMS:
    //ResetBulkEndpoint(IN_ENDPOINT);

    f_RunState = ADC_ST_STOPPED;
    g_ADC_StopReceived = 0;
    DIAG_CLEAR_FLAG(DIAG_DEVSTATE_ADC_STARTED);
#if DEBUG_LED_ADC_RUNNING
    DEBUG_LED_OFF();
#endif

    /*     */
    return CPLD_ADC_Stop();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void ADC_Worker                     /*         USB */
    (void)
    {
    /* !!!    , CPLD  EEPROM !!! */
    /*       -   
        ( USBCmd.c) */

    int bytes_left;
    static unsigned int frame_wpos = 0;

    /*   Stop,     (,   ADC_Stop()   ) */
    if (g_ADC_StopReceived)
        return;
    switch (f_RunState)
        {
        case ADC_ST_RUNNING:
            if (!f_SingleFrame)
                {
                /*       */
                if (f_BufState[f_ReadIdx] == BS_READY)
                    { /*     */
                    if (USB_IN_FREE() >= ADC_BLK_SIZE_B)
                        {
                        WriteUSB((void*)g_Buf.ADC.asRing[f_ReadIdx], ADC_BLK_SIZE_B);
                        f_BufState[f_ReadIdx] = BS_EMPTY; /*     */
                        f_ReadIdx = RING_NEXT(f_ReadIdx, ADC_NBLOCKS); /*   */
                        }
                    }
                }
            break;
        case ADC_ST_FRAME_END: /*     */
            /*   SSC */
            f_StopSSC();
            frame_wpos = 0;
            f_RunState = ADC_ST_FRAME_OUT;
            /*         */
            /* NO break */
        case ADC_ST_FRAME_OUT: /*    USB */
            bytes_left = f_FrameSize - frame_wpos;
            if (bytes_left >= IN_ENDPOINT_SIZE)
                {
                if (USB_IN_FREE() >= IN_ENDPOINT_SIZE)
                    {
                    WriteUSB(((unsigned char*)g_Buf.ADC.asFrame) + frame_wpos, IN_ENDPOINT_SIZE);
                    frame_wpos += IN_ENDPOINT_SIZE;
                    }
                }
            else
                {
                /*  ,  .  . */
                f_RunState = ADC_ST_STOPPED;
                g_ADC_StopReceived = 0;
                DIAG_CLEAR_FLAG(DIAG_DEVSTATE_ADC_STARTED);
#if DEBUG_LED_ADC_RUNNING
                DEBUG_LED_OFF();
#endif
                }
            break;
        } /* switch */
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
