/*================================================================================================*
 *  BOOTROM   (  bootrom   )
 *================================================================================================*/

#include "bootflash.h"
//#include "main.h"
#include "wdt.h"
#include <intrinsics.h>
#include <string.h>

#if BOOTFLASH_ENABLE
/*================================================================================================*/
#define BOOTROM_START       0x00100000  /*    */
#define BOOTROM_HEADER_OFS  0x0080      /*   bootrom header   */
#define BOOTROM_MAXSIZE     0x00003EFF
#define FLASH_PAGE_SIZE     256         /*   FLASH   */
/*================================================================================================*/

/*================================================================================================*/
#pragma pack(1)
typedef struct
    {
    unsigned short BootromVersion;
    unsigned short FlashPageSize;
    unsigned long MaxFirmwareSize;
    char ModuleName[8];
    }
    t_BootromHeader;
#pragma pack()
/*================================================================================================*/

/*================================================================================================*/
static const unsigned char f_BootromCode[] =
    {
#include "bootrom_dump.inc"
    };
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
#pragma diag_suppress=Ta021,Ta023
static __ramfunc void f_ProgramFlash
    (unsigned long LinearAddress)
    {
    __disable_interrupt();
    WDT_Restart();
    while (!(AT91C_BASE_MC->MC_FSR & AT91C_MC_FRDY));
    AT91C_BASE_MC->MC_FCR = AT91C_MC_CORRECT_KEY | AT91C_MC_FCMD_START_PROG |
        (AT91C_MC_PAGEN & LinearAddress);
    while (!(AT91C_BASE_MC->MC_FSR & AT91C_MC_FRDY));
    __enable_interrupt();
    }
#pragma diag_default=Ta021,Ta023
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void BOOTFLASH_UpdateBootrom        /*     */
    (void)
    {
    /* ,     ,      */
    unsigned int Offset;
    unsigned long *FlashPtr, *CodePtr;
    const t_BootromHeader *my_hdr, *present_hdr;

    /*    */
    if (sizeof(f_BootromCode) > BOOTROM_MAXSIZE)
        return;
    if ((unsigned long)&f_BootromCode[0] & 3)
        return;

    /*    */
    my_hdr = (t_BootromHeader*)((const char*)&f_BootromCode + BOOTROM_HEADER_OFS);
    present_hdr = (t_BootromHeader*)(BOOTROM_START + BOOTROM_HEADER_OFS);
    /*     ( ),   .  -   */
    if (!strncmp(my_hdr->ModuleName, present_hdr->ModuleName, sizeof(my_hdr->ModuleName)))
        {
        if (my_hdr->BootromVersion <= present_hdr->BootromVersion)
            return;
        }

    /*  flash */
    FlashPtr = (void*)BOOTROM_START;
    CodePtr = (void*)f_BootromCode;
    for (Offset = 0; Offset < sizeof(f_BootromCode); Offset += FLASH_PAGE_SIZE)
        {
        unsigned int k;
        for (k = 0; k < FLASH_PAGE_SIZE / 4; k++)
            {
            *FlashPtr++ = *CodePtr++;
            }
        f_ProgramFlash((unsigned long)FlashPtr - FLASH_PAGE_SIZE);
        }

    //DIAG_SET_FLAG(0x80000000);

    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
#endif /* BOOTFLASH_ENABLE */
