/*================================================================================================*
 *       
 *================================================================================================*/

#include "bootrom.h"

/*================================================================================================*/
/*   : 0 =  , 1 =   */
#define THIS_IS_BOOTROM     0

#if !THIS_IS_BOOTROM
/*    ( BOOTROM_FLAG == NO_BOOTROM,   "") */
#define NO_BOOTROM          0x45524142
#define BOOTROM_FLAG        (*((volatile const DWORD*)0x00100014))
#endif
/*================================================================================================*/

/*================================================================================================*/
#define VALID_COMMAND       0x01
#define VALID_USBREQ        0x02

#define BOOTSIG_FROM_FW     0x5746  /* "FW",     */
#define BOOTSIG_FROM_BR     0x5242  /* "BR",     */
#if THIS_IS_BOOTROM
#define BOOTSIG_IN          BOOTSIG_FROM_FW
#define BOOTSIG_OUT         BOOTSIG_FROM_BR
#else
#define BOOTSIG_IN          BOOTSIG_FROM_BR
#define BOOTSIG_OUT         BOOTSIG_FROM_FW
#endif
/*================================================================================================*/

/*================================================================================================*/
/*       usb request,
 *       */
typedef struct
    {
    unsigned short Signature;
    unsigned char ValidFields;
    unsigned char Command;
    USBRequest USB_Request;
    }
    t_BootInfo;
#pragma location = "BOOTINFO_SECTION"
static __no_init __root volatile t_BootInfo f_BootInfo;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
int BOOTROM_Present                 /*    */
    (void)                          /*  0  1 */
    {
#if THIS_IS_BOOTROM
    return 1;
#else
    return (BOOTROM_FLAG != NO_BOOTROM);
#endif
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void BOOTROM_Clear                  /*        */
    (void)                          /*    USB    */
    {
    f_BootInfo.Signature = BOOTSIG_OUT;
    f_BootInfo.ValidFields = 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
unsigned char BOOTROM_GetCommand    /*      */
    (void)                          /*     BOOTROM_CMD_NONE */
    {
    if  (
#if !THIS_IS_BOOTROM
        (BOOTROM_FLAG != NO_BOOTROM) &&
#endif
        (f_BootInfo.Signature == BOOTSIG_IN) &&
        (f_BootInfo.ValidFields & VALID_COMMAND)
        )
        {
        return f_BootInfo.Command;
        }
    else
        {
        return BOOTCMD_NONE;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int BOOTROM_GetRequest              /*   USB     */
    (
    USBRequest *pRequest            /* () -    */
    )                               /*  1,    ,  0 */
    {
    if  (
#if !THIS_IS_BOOTROM
        (BOOTROM_FLAG != NO_BOOTROM) &&
#endif
        (f_BootInfo.Signature == BOOTSIG_IN) &&
        (f_BootInfo.ValidFields & VALID_USBREQ)
        && pRequest
        )
        {
        memcpy(pRequest, (void*)&f_BootInfo.USB_Request, sizeof(USBRequest));
        return 1;
        }
    else
        {
        return 0;
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void BOOTROM_SetCommand             /*      */
    (
    unsigned char Command           /* ()  -  (BOOTROM_CMD_...) */
    )                               /*     */
    {
#if !THIS_IS_BOOTROM
    if (BOOTROM_FLAG == NO_BOOTROM)
        return;
#endif
    if (f_BootInfo.Signature != BOOTSIG_OUT)
        BOOTROM_Clear();
    f_BootInfo.ValidFields |= VALID_COMMAND;
    f_BootInfo.Command = Command;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void BOOTROM_SetRequest             /*  USB     */
    (
    const USBRequest *pRequest      /* ()  -     */
    )                               /*      */
    {
#if !THIS_IS_BOOTROM
    if (BOOTROM_FLAG == NO_BOOTROM)
        return;
#endif
    if (f_BootInfo.Signature != BOOTSIG_OUT)
        BOOTROM_Clear();
    if (pRequest)
        {
        f_BootInfo.ValidFields |= VALID_USBREQ;
        memcpy((void*)&f_BootInfo.USB_Request, pRequest, sizeof(USBRequest));
        }
    else
        {
        f_BootInfo.ValidFields &= ~VALID_USBREQ;
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
