/*================================================================================================*
 *      Altera
 * (USART0, ,  SCK (8 ), 9bitTX/8bitRX, MSB first, 1 stop, no parity)
 *================================================================================================*/

#include "cpld.h"
#include "timers.h"
#include "wdt.h"
#include "pio.h"
#include <stdlib.h>

/*================================================================================================*/
/*   */
#define CMD_ADC_PREPARE         0x101
#define CMD_ADC_STARTSTOP       0x102
#define CMD_ADC_SETFRAMEDELAY   0x103
#define CMD_CFG_SETCFG          0x104
#define CMD_ADC_SETFREQ         0x105
#define CMD_TTL_INOUT           0x106
#define CMD_DAC_CTL             0x107
#define CMD_DAC_STARTSTOP       0x108
#define CMD_ADC_SETANALOGCHAN   0x109
#define CMD_ADC_SETANALOGLEVEL  0x10A
#define CMD_CFG_GETVERSION      0x10E

typedef volatile unsigned short int t_DMA_Data;

static unsigned int f_Timeout = 0;
static unsigned short int f_LastTTLOut = 0;
static unsigned char f_LastConfigReg = 0;
static unsigned char f_CfgValid = 0;
static unsigned char f_LastDACFreqDiv = 0;
static short int f_ChipRevision = -1;

const short int *const g_CPLD_ChipRevisionPtr = &f_ChipRevision;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
static int f_GenericCommand         /*     () */
    (
    unsigned int RequestLen,        /* ()  -      */
    t_DMA_Data* RequestBuf,         /* ()  -      */
    unsigned int ReplyLen,          /* ()  -    */
    t_DMA_Data* ReplyBuf            /* () -       */
    )                               /*  1  , 0   */
    {
    unsigned int xfer_ok, end_flags;

    /*   */
    if ((RequestLen == 0) || !RequestBuf)
        return 0;
    if (ReplyLen && !ReplyBuf)
        return 0;

    /*    ,   */
    if (!CPLD_PowerGood())
        return 0;

    /*     */
    /*   , ..     ""  */
    AT91C_BASE_US0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTSTA;
    AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;
    if (AT91C_BASE_US0->US_RHR);    /*    -  RXRDY */

    /*   PDC */
    AT91C_BASE_US0->US_TPR = (unsigned int)RequestBuf;
    AT91C_BASE_US0->US_TCR = RequestLen;
    AT91C_BASE_US0->US_TNCR = 0;
    AT91C_BASE_US0->US_RPR = (unsigned int)ReplyBuf;
    AT91C_BASE_US0->US_RCR = ReplyLen;
    AT91C_BASE_US0->US_RNCR = 0;

    /*  DMA- PDC */
    if (ReplyLen > 0)
        { /*    */
        AT91C_BASE_US0->US_PTCR = AT91C_PDC_RXTEN | AT91C_PDC_TXTEN;
        end_flags = AT91C_US_ENDTX | AT91C_US_ENDRX;
        }
    else
        { /*   */
        AT91C_BASE_US0->US_PTCR = AT91C_PDC_TXTEN;
        end_flags = AT91C_US_ENDTX;
        }

    xfer_ok = 0; /*   */
    if (f_Timeout > 0)
        {
        TIMER_StartTimeout(f_Timeout);
        while (!xfer_ok && !TIMER_TimeoutExpired())
            {
            WDT_Restart();
            xfer_ok = ((AT91C_BASE_US0->US_CSR & end_flags) == end_flags);
            }
        }
    else
        {
        WDT_Restart();
        while (!xfer_ok)
            {
            xfer_ok = ((AT91C_BASE_US0->US_CSR & end_flags) == end_flags);
            }
        }

    /*  PDC */
    AT91C_BASE_US0->US_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;

    /*  1,  ,  0   */
    return xfer_ok;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CPLD_InitUSART                 /*     USART */
    (void)
    {
    /*  USART clock */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_US0);

    /*  USART */
    AT91C_BASE_US0->US_CR =
        AT91C_US_STPBRK | AT91C_US_RSTRX | AT91C_US_RSTTX |
        AT91C_US_RXDIS | AT91C_US_TXDIS | AT91C_US_RSTSTA;

    AT91C_BASE_US0->US_MR =
        AT91C_US_USMODE_NORMAL |    /* Normal USART mode */
        AT91C_US_CLKS_EXT |         /* Use SCK as clock */
        AT91C_US_CHRL_8_BITS |      /* 8-bit mode */
        AT91C_US_SYNC |             /* synchronous mode */
        AT91C_US_PAR_NONE |         /* no parity */
        AT91C_US_NBSTOP_1_BIT |     /* 1 stop bit */
        AT91C_US_CHMODE_NORMAL |    /* no loopback */
        AT91C_US_MSBF |             /* MSB first */
        AT91C_US_MODE9;             /* 9-bit (overrides CHRL) */

    AT91C_BASE_US0->US_IDR = 0xFFFFFFFF; /*     USART */
    if (AT91C_BASE_US0->US_RHR);    /*    -  RXRDY */
    AT91C_BASE_US0->US_TTGR = 0;
    AT91C_BASE_US0->US_RTOR = 0;
    AT91C_BASE_US0->US_BRGR = 0;

    /*  PDC  USART */
    AT91C_BASE_US0->US_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;
    AT91C_BASE_US0->US_RCR = 0;
    AT91C_BASE_US0->US_RNCR = 0;
    AT91C_BASE_US0->US_TCR = 0;
    AT91C_BASE_US0->US_TNCR = 0;

    /*     */
    AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

    /*   TXD, RXD, SCK  USART */
    AT91C_BASE_PIOA->PIO_ASR = AT91C_PA5_RXD0 | AT91C_PA6_TXD0;
    AT91C_BASE_PIOA->PIO_BSR = AT91C_PA2_SCK0;
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA2_SCK0 | AT91C_PA5_RXD0 | AT91C_PA6_TXD0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_PowerGood                  /*        */
    (void)                          /*  1 =   ; 0 =    */
    {
    /*   N_RESET   ( -  ),     */
    /*   LOW_POWER   ,       */
    if ((AT91C_BASE_PIOA->PIO_OSR & PIO_N_RESET) || (AT91C_BASE_PIOA->PIO_ODSR & PIO_LOW_POWER))
        return 0;
    /*   N_RESET  ,       = power good */
    return ((AT91C_BASE_PIOA->PIO_PDSR & PIO_N_RESET) != 0);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_Initialize                 /*  */
    (
    unsigned int InitConfigState    /* ()  -      */
    )                               /*  1  , 0   */
    {
    int ok;
    t_DMA_Data cmd[3];
    t_DMA_Data reply[2];

    f_ChipRevision = -1;

    /*   
       ( ,  SetConfigBits(CPLD_CONFIG_VALID_BITS, InitConfigState)) */
    cmd[0] = CMD_CFG_SETCFG;
    cmd[1] = InitConfigState & CPLD_CONFIG_VALID_BITS;
    f_Timeout = CPLD_POWER_ON_TIMEOUT;
    ok = f_GenericCommand(2, cmd, 0, NULL);
    f_Timeout = CPLD_COMMAND_TIMEOUT;
    if (!ok)
        return 0;
    f_LastConfigReg = cmd[1];
    f_CfgValid = 1;

    /*   TTL In/Out (   ).
             (   TTL)   0,
             ,       .
         ,    "-1" (). */
    f_LastTTLOut = 0;
    cmd[0] = CMD_TTL_INOUT;
    cmd[1] = (f_LastTTLOut >> 8) & 0xFF; /*   */
    cmd[2] = f_LastTTLOut & 0xFF; /*   */
    if (!f_GenericCommand(3, cmd, 2, reply))
        return 0;
    if ((reply[0] & 0x01) == 0)
        { /*    */
        cmd[0] = CMD_CFG_GETVERSION;
        cmd[1] = 0; /*   :  2  */
        if (!f_GenericCommand(2, cmd, 1, reply))
            return 0;
        f_ChipRevision = reply[0] ^ 0x1FF; /*    reply[0]  */
        }

    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_PowerOn                    /*        */
    (
    unsigned int Timeout            /* ()  -    ( ~1000, 0 = ) */
    )                               /*  ,  0   */
    {
    f_CfgValid = 0;
    /*  RESET (active low)   */
    AT91C_BASE_PIOA->PIO_CODR = PIO_N_RESET;
    AT91C_BASE_PIOA->PIO_OER = PIO_N_RESET;
    /*    */
    AT91C_BASE_PIOA->PIO_CODR = PIO_LOW_POWER;
    __asm("nop");
    __asm("nop");
    /*   ,   N_RESET   */
    AT91C_BASE_PIOA->PIO_ODR = PIO_N_RESET;
    /*     .   Power Good. */
    if (Timeout)
        {
        TIMER_StartTimeout(Timeout);
        while (!CPLD_PowerGood())
            {
            if (TIMER_TimeoutExpired())
                return 0;
            WDT_Restart();
            }
        }
    else
        {
        WDT_Restart(); /*  watchdog   */
        /*    (   WDT) */
        while (!CPLD_PowerGood());
        }
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CPLD_PowerOff                  /*        */
    (void)
    {
    f_CfgValid = 0;
    /*  RESET (active low)   */
    AT91C_BASE_PIOA->PIO_CODR = PIO_N_RESET;
    AT91C_BASE_PIOA->PIO_OER = PIO_N_RESET;
    /*       */
    AT91C_BASE_PIOA->PIO_SODR = PIO_LOW_POWER;
    __asm("nop");
    __asm("nop");
    /* Reset   */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CPLD_SetTimeout                /*      */
    (
    unsigned int Timeout            /* ()  -    ( ~1000, 0 = ) */
    )
    {
    f_Timeout = Timeout;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
unsigned int CPLD_CFG_GetLastConfig /*       */
    (void)                          /*    ( CPLD_CONFIG_...) */
    {
    return (f_CfgValid) ? f_LastConfigReg : CPLD_CONFIG_RESET_STATE;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_CFG_SetConfigBits          /*     */
    (
    unsigned int RegMask,           /* ()  -    ( CPLD_CONFIG_...) */
    unsigned int RegValue           /* ()  -    */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    cmd[0] = CMD_CFG_SETCFG;
    if (f_CfgValid)
        {
        cmd[1] = ((f_LastConfigReg & ~RegMask) | (RegValue & RegMask)) & CPLD_CONFIG_VALID_BITS;
        if (cmd[1] == f_LastConfigReg)
            return 1;
        }
    else
        {
        cmd[1] = ((CPLD_CONFIG_RESET_STATE & ~RegMask) | (RegValue & RegMask))
            & CPLD_CONFIG_VALID_BITS;
        }
    if (f_GenericCommand(2, cmd, 0, NULL))
        {
        f_LastConfigReg = cmd[1];
        f_CfgValid = 1;
        return 1;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_InitAndPreload         /*    */
    (
    unsigned int Mode               /* ()  -  (  CPLD_ADCMODE_...) */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    t_DMA_Data reply[1];
    cmd[0] = CMD_ADC_PREPARE;
    cmd[1] = Mode & CPLD_ADCMODE_VALID_BITS;
    if (!f_GenericCommand(2, cmd, 1, reply))
        return 0;
    return (reply[0] == 0x1FF);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetFrameDelay          /*     */
    (
    unsigned int CycleCount         /* ()  -    , 0..255 */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    if (CycleCount > 0xFF)
        return 0;
    cmd[0] = CMD_ADC_SETFRAMEDELAY;
    cmd[1] = CycleCount & 0xFF;
    return f_GenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetSamplingRate        /*     */
    (
    unsigned int FreqDiv            /* ()  -  , 16  (F = 8000000/(DIV+1)) */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[3];
    if (FreqDiv > 0xFFFF)
        return 0;
    cmd[0] = CMD_ADC_SETFREQ;
    cmd[1] = (FreqDiv >> 8) & 0xFF;
    cmd[2] = FreqDiv & 0xFF;
    return f_GenericCommand(3, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetupAnalogSync        /*    */
    (
    unsigned int ChannelSelect,     /* ()  -   ( ) */
    int ThresholdValue              /* ()  -   */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[3];
    cmd[0] = CMD_ADC_SETANALOGCHAN;
    cmd[1] = ChannelSelect & 0xFF;
    if (!f_GenericCommand(2, cmd, 0, NULL))
        return 0;
    cmd[0] = CMD_ADC_SETANALOGLEVEL;
    cmd[1] = (ThresholdValue >> 8) & 0xFF;
    cmd[2] = ThresholdValue & 0xFF;
    return f_GenericCommand(3, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_Start                  /*   */
    (
    unsigned int Mode               /*   (CPLD_ADCSTART_...) */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    cmd[0] = CMD_ADC_STARTSTOP;
    cmd[1] = Mode & 0xFF;
    return f_GenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_Stop                   /*   */
    (void)                          /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    cmd[0] = CMD_ADC_STARTSTOP;
    cmd[1] = CPLD_ADCSTART_STOP;
    return f_GenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Async                  /*      */
    (
    unsigned int DAC_Command        /* ()  -  (CPLD_DACCTL_ASYNC_...) */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    t_DMA_Data reply[1];
    cmd[0] = CMD_DAC_CTL;
    cmd[1] = DAC_Command & 0xFF;
    return (f_GenericCommand(2, cmd, 1, reply) && (reply[0] == 0x1FF));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Start                  /*   */
    (
    unsigned int SyncWithADC,       /* 0 =  , 1 =      */
    unsigned int FreqDiv            /*   (0..7, F=200kHz/(N+1)) */
    )                               /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    f_LastDACFreqDiv = (FreqDiv << CPLD_DAC_FREQDIV_SHIFT) & CPLD_DAC_FREQDIV_MASK;
    cmd[0] = CMD_DAC_STARTSTOP;
    cmd[1] = CPLD_DAC_START | ((SyncWithADC) ? CPLD_DAC_SYNC_WITH_ADC : 0) | f_LastDACFreqDiv;
    return f_GenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Stop                   /*   */
    (void)                          /*  1  , 0   */
    {
    t_DMA_Data cmd[2];
    t_DMA_Data reply[1];
    cmd[0] = CMD_DAC_STARTSTOP;
    cmd[1] = CPLD_DAC_STOP | f_LastDACFreqDiv;
    /*   ,    ("-1"),    
          .      . */
    return f_GenericCommand(2, cmd, (f_ChipRevision >= 0) /* 0  1   */, reply);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_TTL_In                     /*    DI 0..15 */
    (
    unsigned short int *pValue      /* () -    -  */
    )                               /*  1  , 0   */
    {
    /*  16    DINx  .
     *        ,   
     *     f_ttl_cmd (  cn_USB_TTLOut()).
     */
    t_DMA_Data cmd[3];
    t_DMA_Data reply[2];
    cmd[0] = CMD_TTL_INOUT;
    cmd[1] = (f_LastTTLOut >> 8) & 0xFF; /*   */
    cmd[2] = f_LastTTLOut & 0xFF; /*   */
    if (f_GenericCommand(3, cmd, 2, reply))
        {
        /*   9-  reply[]  */
        /* reply[0] =   << 1 */
        /* reply[1] =   << 1 */
        unsigned short hi = reply[0] >> 1;
        *pValue = (hi << 8) | (reply[1] >> 1);
        return 1;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_TTL_Out                    /*    DOUT 0..15 */
    (
    unsigned short int Value        /* ()  -   (16 ) */
    )                               /*  1  , 0   */
    {
    /*  16    DOUTx  .
     *       f_ttl_cmd,  
     *     (..        
     *    ).
     *   DINx .
     */
    t_DMA_Data cmd[3];
    t_DMA_Data reply[2];
    f_LastTTLOut = Value;
    cmd[0] = CMD_TTL_INOUT;
    cmd[1] = (f_LastTTLOut >> 8) & 0xFF; /*   */
    cmd[2] = f_LastTTLOut & 0xFF; /*   */
    return f_GenericCommand(3, cmd, 2, reply);
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
