/*================================================================================================*
 *      Altera
 * (USART0, ,  SCK (8 ), 9bitTX/8bitRX, MSB first, 1 stop, no parity)
 *================================================================================================*/

#include "cpld.h"
#include "timers.h"
#include "wdt.h"
#include "pio.h"
#include <stdlib.h>

/*================================================================================================*/
/* 16-        DMA */
static volatile  unsigned short int f_xmit_buf[4];
static volatile unsigned short int f_recv_buf[4];

static unsigned int f_Timeout = 0;
static unsigned int f_ADC_LastMode = 0;
static unsigned int f_LastConfigReg = 0;
static int f_CfgValid = 0;

/*   /  / */
static unsigned char f_ttl_cmd[3];
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void CPLD_InitUSART                 /*     USART */
    (void)
    {
    /*  USART clock */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_US0);

    /*  USART */
    AT91C_BASE_US0->US_CR =
        AT91C_US_STPBRK | AT91C_US_RSTRX | AT91C_US_RSTTX |
        AT91C_US_RXDIS | AT91C_US_TXDIS | AT91C_US_RSTSTA;

    AT91C_BASE_US0->US_MR =
        AT91C_US_USMODE_NORMAL |    /* Normal USART mode */
        AT91C_US_CLKS_EXT |         /* Use SCK as clock */
        AT91C_US_CHRL_8_BITS |      /* 8-bit mode */
        AT91C_US_SYNC |             /* synchronous mode */
        AT91C_US_PAR_NONE |         /* no parity */
        AT91C_US_NBSTOP_1_BIT |     /* 1 stop bit */
        AT91C_US_CHMODE_NORMAL |    /* no loopback */
        AT91C_US_MSBF |             /* MSB first */
        AT91C_US_MODE9;             /* 9-bit (overrides CHRL) */

    AT91C_BASE_US0->US_IDR = 0xFFFFFFFF; /*     USART */
    if (AT91C_BASE_US0->US_RHR);    /*    -  RXRDY */
    AT91C_BASE_US0->US_TTGR = 0;
    AT91C_BASE_US0->US_RTOR = 0;
    AT91C_BASE_US0->US_BRGR = 0;

    /*  PDC  USART */
    AT91C_BASE_US0->US_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;
    AT91C_BASE_US0->US_RCR = 0;
    AT91C_BASE_US0->US_RNCR = 0;
    AT91C_BASE_US0->US_TCR = 0;
    AT91C_BASE_US0->US_TNCR = 0;

    /*     */
    AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;

    /*   TXD, RDX, SCK  USART */
    AT91C_BASE_PIOA->PIO_ASR = AT91C_PA5_RXD0 | AT91C_PA6_TXD0;
    AT91C_BASE_PIOA->PIO_BSR = AT91C_PA2_SCK0;
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA2_SCK0 | AT91C_PA5_RXD0 | AT91C_PA6_TXD0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_PowerGood                  /*        */
    (void)                          /*  1 =   ; 0 =    */
    {
    /*   N_RESET   ( -  ),     */
    /*   LOW_POWER   ,       */
    if ((AT91C_BASE_PIOA->PIO_OSR & PIO_N_RESET) || (AT91C_BASE_PIOA->PIO_ODSR & PIO_LOW_POWER))
        return 0;
    /*   N_RESET  ,       = power good */
    return ((AT91C_BASE_PIOA->PIO_PDSR & PIO_N_RESET) != 0);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_PowerOn                    /*        */
    (
    unsigned int Timeout            /* ()  -    ( ~1000, 0 = ) */
    )                               /*  ,  0   */
    {
    f_CfgValid = 0;
    /*  RESET (active low)   */
    AT91C_BASE_PIOA->PIO_CODR = PIO_N_RESET;
    AT91C_BASE_PIOA->PIO_OER = PIO_N_RESET;
    /*    */
    AT91C_BASE_PIOA->PIO_CODR = PIO_LOW_POWER;
    __asm("nop");
    __asm("nop");
    /*   ,   N_RESET   */
    AT91C_BASE_PIOA->PIO_ODR = PIO_N_RESET;
    /*     .   Power Good. */
    if (Timeout)
        {
        TIMER_StartTimeout(Timeout);
        while (!CPLD_PowerGood())
            {
            if (TIMER_TimeoutExpired())
                return 0;
            WDT_Restart();
            }
        }
    else
        {
        WDT_Restart(); /*  watchdog   */
        /*    (   WDT) */
        while (!CPLD_PowerGood());
        }
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CPLD_PowerOff                  /*        */
    (void)
    {
    f_CfgValid = 0;
    /*  RESET (active low)   */
    AT91C_BASE_PIOA->PIO_CODR = PIO_N_RESET;
    AT91C_BASE_PIOA->PIO_OER = PIO_N_RESET;
    /*       */
    AT91C_BASE_PIOA->PIO_SODR = PIO_LOW_POWER;
    __asm("nop");
    __asm("nop");
    /* Reset   */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void CPLD_SetTimeout                /*      */
    (
    unsigned int Timeout            /* ()  -    ( ~1000, 0 = ) */
    )
    {
    f_Timeout = Timeout;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_RunGenericCommand          /*     () */
    (
    unsigned int CommandLen,        /* ()  -      */
    unsigned char *CommandBuffer,   /* ()  -      */
    unsigned int ReplyLen,          /* ()  -    */
    unsigned char *ReplyBuffer      /* () -       */
    )                               /*  1  , 0   */
    {
    unsigned int k, xfer_ok, end_flags;

    /*   */
    if (!(CommandLen && CommandBuffer))
        return 0;
    if (ReplyLen && !ReplyBuffer)
        return 0;
    if (CommandLen > sizeof(f_xmit_buf)/sizeof(f_xmit_buf[0]))
        return 0;
    if (ReplyLen > sizeof(f_recv_buf)/sizeof(f_recv_buf[0]))
        return 0;

    /*    ,   */
    if (!CPLD_PowerGood())
        return 0;

    /*       16  ( 8 = "1"    ) */
    f_xmit_buf[0] = 0x100 | CommandBuffer[0];
    for (k = 1; k < CommandLen; k++)
        {
        f_xmit_buf[k] = CommandBuffer[k];
        }

    /*     */
    /*   , ..     ""  */
    AT91C_BASE_US0->US_CR = AT91C_US_RSTRX | AT91C_US_RSTSTA;
    AT91C_BASE_US0->US_CR = AT91C_US_RXEN | AT91C_US_TXEN;
    if (AT91C_BASE_US0->US_RHR);    /*    -  RXRDY */

    /*   PDC */
    AT91C_BASE_US0->US_TPR = (unsigned int)&f_xmit_buf;
    AT91C_BASE_US0->US_TCR = CommandLen;
    AT91C_BASE_US0->US_TNCR = 0;
    AT91C_BASE_US0->US_RPR = (unsigned int)&f_recv_buf;
    AT91C_BASE_US0->US_RCR = ReplyLen;
    AT91C_BASE_US0->US_RNCR = 0;

    /*  DMA- PDC */
    if (ReplyLen > 0)
        { /*    */
        AT91C_BASE_US0->US_PTCR = AT91C_PDC_RXTEN | AT91C_PDC_TXTEN;
        end_flags = AT91C_US_ENDTX | AT91C_US_ENDRX;
        }
    else
        { /*   */
        AT91C_BASE_US0->US_PTCR = AT91C_PDC_TXTEN;
        end_flags = AT91C_US_ENDTX;
        }

    xfer_ok = 0; /*   */
    if (f_Timeout > 0)
        {
        TIMER_StartTimeout(f_Timeout);
        while (!xfer_ok && !TIMER_TimeoutExpired())
            {
            WDT_Restart();
            xfer_ok = ((AT91C_BASE_US0->US_CSR & end_flags) == end_flags);
            }
        }
    else
        {
        WDT_Restart();
        while (!xfer_ok)
            {
            xfer_ok = ((AT91C_BASE_US0->US_CSR & end_flags) == end_flags);
            }
        }

    /*  PDC */
    AT91C_BASE_US0->US_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;

    /*   ,     */
    /*   8..1,      8-    ,
          ()   -,   "1" */
    if (xfer_ok)
        {
        for (k = 0; k < ReplyLen; k++)
            {
            ReplyBuffer[k] = (f_recv_buf[k] >> 1) & 0xFF;
            }
        }

    /*  1,  ,  0   */
    return xfer_ok;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*  :         .. */
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
unsigned int CPLD_CFG_GetLastConfig /*       */
    (void)                          /*    ( CPLD_CONFIG_...) */
    {
    return (f_CfgValid) ? f_LastConfigReg : CPLD_CONFIG_RESET_STATE;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_CFG_SetConfigBits          /*     */
    (
    unsigned int RegMask,           /* ()  -    ( CPLD_CONFIG_...) */
    unsigned int RegValue           /* ()  -    */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[2];
    cmd[0] = 0x04;
    if (f_CfgValid)
        {
        cmd[1] = ((f_LastConfigReg & ~RegMask) | (RegValue & RegMask)) & CPLD_CONFIG_VALID_BITS;
        if (cmd[1] == f_LastConfigReg)
            return 1;
        }
    else
        {
        cmd[1] = ((CPLD_CONFIG_RESET_STATE & ~RegMask) | (RegValue & RegMask))
            & CPLD_CONFIG_VALID_BITS;
        }
    if (CPLD_RunGenericCommand(2, cmd, 0, NULL))
        {
        f_LastConfigReg = cmd[1];
        f_CfgValid = 1;
        return 1;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_InitAndPreload         /*    */
    (
    unsigned int Mode               /* ()  -  (  CPLD_ADCMODE_...) */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[2];
    unsigned char reply;
    cmd[0] = 0x01;
    cmd[1] = Mode & CPLD_ADCMODE_VALID_BITS;
    if (!CPLD_RunGenericCommand(2, cmd, 1, &reply))
        return 0;
    if (reply == 0xFF)
        {
        f_ADC_LastMode = Mode;
        return 1;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
unsigned int CPLD_ADC_GetLastMode   /*      */
    (void)                          /*   (  CPLD_ADCMODE_...) */
    {
    return f_ADC_LastMode;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetFrameDelay          /*     */
    (
    unsigned int CycleCount         /* ()  -    , 0..255 */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[2];
    if (CycleCount > 0xFF)
        return 0;
    cmd[0] = 0x03;
    cmd[1] = CycleCount & 0xFF;
    return CPLD_RunGenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetSamplingRate        /*     */
    (
    unsigned int FreqDiv            /* ()  -  , 16  (F = 8000000/(DIV+1)) */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[3];
    if (FreqDiv > 0xFFFF)
        return 0;
    cmd[0] = 0x05;
    cmd[1] = (FreqDiv >> 8) & 0xFF;
    cmd[2] = FreqDiv & 0xFF;
    return CPLD_RunGenericCommand(3, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_SetupAnalogSync        /*    */
    (
    unsigned int ChannelSelect,     /* ()  -   ( ) */
    int ThresholdValue              /* ()  -   */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[3];
    cmd[0] = 0x09;
    cmd[1] = ChannelSelect;
    if (!CPLD_RunGenericCommand(2, cmd, 0, NULL))
        return 0;
    cmd[0] = 0x0A;
    cmd[2] = ThresholdValue & 0xFF;
    cmd[1] = (ThresholdValue >> 8) & 0xFF;
    return CPLD_RunGenericCommand(3, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_Start                  /*   */
    (
    unsigned int Mode               /*   (CPLD_ADCSTART_...) */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[2];
    cmd[0] = 0x02;
    cmd[1] = Mode;
    return CPLD_RunGenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_ADC_Stop                   /*   */
    (void)                          /*  1  , 0   */
    {
    unsigned char cmd[2];
    cmd[0] = 0x02;
    cmd[1] = CPLD_ADCSTART_STOP;
    return CPLD_RunGenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Async                  /*      */
    (
    unsigned int DAC_Command        /* ()  -  (CPLD_DACCTL_ASYNC_...) */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[2];
    unsigned char reply;
    cmd[0] = 0x07;
    cmd[1] = DAC_Command;
    return (CPLD_RunGenericCommand(2, cmd, 1, &reply) && (reply == 0xFF));
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Start                  /*   */
    (
    unsigned int SyncWithADC,       /* 0 =  , 1 =      */
    unsigned int FreqDiv            /*   (0..7, F=200kHz/(N+1)) */
    )                               /*  1  , 0   */
    {
    unsigned char cmd[2];
    cmd[0] = 0x08;
    cmd[1] = CPLD_DAC_START | ((SyncWithADC) ? CPLD_DAC_SYNC_WITH_ADC : 0)
        | ((FreqDiv << CPLD_DAC_FREQDIV_SHIFT) & CPLD_DAC_FREQDIV_MASK);
    return CPLD_RunGenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_DAC_Stop                   /*   */
    (void)                          /*  1  , 0   */
    {
    unsigned char cmd[2];
    cmd[0] = 0x08;
    cmd[1] = CPLD_DAC_STOP;
    return CPLD_RunGenericCommand(2, cmd, 0, NULL);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_TTL_In                     /*    DI 0..15 */
    (
    unsigned short int *pValue      /* () -    -  */
    )                               /*  1  , 0   */
    {
    /*  16    DINx  .
     *        ,   
     *     f_ttl_cmd (  cn_USB_TTLOut()).
     */
    unsigned char reply[2];
    f_ttl_cmd[0] = 0x06; /*  IN/OUT */
    if (CPLD_RunGenericCommand(3, f_ttl_cmd, 2, reply))
        {
        *pValue = ((unsigned short int)reply[0] << 8) | reply[1]; /*     */
        return 1;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int CPLD_TTL_Out                    /*    DOUT 0..15 */
    (
    unsigned short int Value        /* ()  -   (16 ) */
    )                               /*  1  , 0   */
    {
    /*  16    DOUTx  .
     *       f_ttl_cmd,  
     *     (..        
     *    ).
     *   DINx .
     */
    unsigned char reply[2];
    f_ttl_cmd[0] = 0x06; /*  IN/OUT */
    f_ttl_cmd[1] = (Value >> 8) & 0xFF; /*   */
    f_ttl_cmd[2] = Value & 0xFF; /*   */
    return CPLD_RunGenericCommand(3, f_ttl_cmd, 2, reply);
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
