/*================================================================================================*
 *  SPI (, slave  EEPROM, master)
 *================================================================================================*/

#include "spi.h"
#include "clk.h"

/*================================================================================================*/
#define SPI_MASTER_CLK      4000000 /*  SPI    EEPROM AT25010 */
#define SPI_CLKDIV          (CLK_MCK_FREQ / SPI_MASTER_CLK)
#if ((SPI_CLKDIV < 1) || (SPI_CLKDIV > 255))
#error "Invalid SPI master clock"
#endif
/*================================================================================================*/

/*================================================================================================*/
static int f_mode = SPI_MODE_CLOSED;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void SPI_Init                       /*   SPI */
    (void)
    {
    SPI_Close();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void SPI_Close                      /*  SPI      Z */
    (void)
    {
    /*  SPI   DMA  */
    AT91C_BASE_SPI->SPI_IDR = 0xFFFFFFFF;
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SPIDIS;
    AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;

    /*     SPI     ( NPCS3,   1) */
    AT91C_BASE_PIOA->PIO_ODR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK;
    AT91C_BASE_PIOA->PIO_SODR = AT91C_PA22_NPCS3;
    AT91C_BASE_PIOA->PIO_OER = AT91C_PA22_NPCS3;
    AT91C_BASE_PIOA->PIO_PER = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK |
                               AT91C_PA22_NPCS3;

    /*   SPI */
    AT91C_BASE_PMC->PMC_PCDR = (1 << AT91C_ID_SPI);

    f_mode = SPI_MODE_CLOSED;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int SPI_IsOpenAs                    /*    SPI */
    (
    int Mode                        /* ()  -    (SPI_MODE_...) */
    )                               /*  1,     */
    {
    return (f_mode == Mode);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int SPI_OpenForDAC                  /*  SPI   (  slave) */
    (void)                          /*  1    0,  SPI   */
    {
    if (f_mode != SPI_MODE_CLOSED)
        return 0;

    /*   SPI */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_SPI);

    /*    */
    AT91C_BASE_SPI->SPI_IDR = 0xFFFFFFFF;

    /*  DMA */
    AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;

    /*  SPI     */
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SWRST; /* reset ( => slave mode) */
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SWRST; /* reset ( => slave mode) */
    AT91C_BASE_SPI->SPI_CSR[0] =
        (AT91C_SPI_CPOL & 0) |      /* CPOL = 0 */
        AT91C_SPI_NCPHA |           /* NCPHA = 1 */
        AT91C_SPI_BITS_16 |         /* 16 bit mode */
        (AT91C_SPI_SCBR & (1 << 8));
    AT91C_BASE_SPI->SPI_MR =
        (AT91C_SPI_MSTR & 0) |      /* Slave */
        AT91C_SPI_PS_FIXED |        /* Fixed peripheral select */
        (AT91C_SPI_PCSDEC & 0) |    /* No binary decoder on NPCSx */
        AT91C_SPI_MODFDIS |         /* Mode fault detect disable */
        (AT91C_SPI_PCS & (0x0E << 16)) | /* PCS = 1110 = CS0 */
        (AT91C_SPI_DLYBCS & (0 << 24)); /* Delay between chip selects */
    if (AT91C_BASE_SPI->SPI_RDR); /*   RDRF (-    1) */

    /* NPCS3    ,    "1" ( EEPROM) */
    AT91C_BASE_PIOA->PIO_SODR = AT91C_PA22_NPCS3;
    AT91C_BASE_PIOA->PIO_OER = AT91C_PA22_NPCS3;
    AT91C_BASE_PIOA->PIO_PER = AT91C_PA22_NPCS3;
    /*     SPI */
    AT91C_BASE_PIOA->PIO_ASR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK | AT91C_PA11_NPCS0;
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK | AT91C_PA11_NPCS0;

    f_mode = SPI_MODE_SLAVE;
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int SPI_OpenForEEPROM               /*  SPI    EEPROM */
    (void)                          /*  1    0,  SPI   */
    /* : ,       SPI- */
    {
    if (f_mode != SPI_MODE_CLOSED)
        return 0;

    /*   SPI */
    AT91C_BASE_PMC->PMC_PCER = (1 << AT91C_ID_SPI);

    /*  EEPROM DMA   */
    AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_RXTDIS | AT91C_PDC_TXTDIS;

    /*  SPI    AT25010 */
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SWRST; /* reset ( => slave mode) */
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SWRST; /* reset ( => slave mode) */
    AT91C_BASE_SPI->SPI_CSR[3] =
        (AT91C_SPI_CPOL & 0) |      /* CPOL = 0 */
        AT91C_SPI_NCPHA |           /* NCPHA = 1 */
        AT91C_SPI_CSAAT |           /* NPCS3 remains low after transfer */
        AT91C_SPI_BITS_8 |          /* 8 bit mode */
        (AT91C_SPI_SCBR & (SPI_CLKDIV << 8)) |
        (AT91C_SPI_DLYBS & (0 << 16)) | /* Delay between NPCS and SPCK (0 = 1/2 clock period) */
        (AT91C_SPI_DLYBCT & (0 << 24)); /* Delay between transfers */
    AT91C_BASE_SPI->SPI_MR =
        AT91C_SPI_MSTR |            /* Master */
        AT91C_SPI_PS_FIXED |        /* Fixed peripheral select */
        (AT91C_SPI_PCSDEC & 0) |    /* No binary decoder on NPCSx */
        AT91C_SPI_MODFDIS |         /* Mode fault detect disable */
        (AT91C_SPI_PCS & (0x07 << 16)) | /* PCS = 0111 = NPCS3 */
        (AT91C_SPI_DLYBCS & (0 << 24)); /* Delay between chip selects */
    AT91C_BASE_SPI->SPI_IDR = 0xFFFFFFFF; /*    */
    if (AT91C_BASE_SPI->SPI_RDR); /*   RDRF (-    1) */

    /*     SPI */
    AT91C_BASE_PIOA->PIO_ASR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK;
    AT91C_BASE_PIOA->PIO_BSR = AT91C_PA22_NPCS3;
    AT91C_BASE_PIOA->PIO_PDR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK |
                               AT91C_PA22_NPCS3;
    /*  SPI */
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SPIEN;

    f_mode = SPI_MODE_MASTER;
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void SPI_StartDMATransmit           /*    DMA ( ) */
    (                               /*  DMA     */
    int EnableDMAInterrupt
    )
    {
    if (f_mode != SPI_MODE_SLAVE)
        return;

    /*   */
    if (EnableDMAInterrupt)
        AT91C_BASE_SPI->SPI_IER = AT91C_SPI_ENDTX;
    /*  PDC   */
    AT91C_BASE_SPI->SPI_PTCR = AT91C_PDC_TXTEN;
    /*  SPI */
    AT91C_BASE_SPI->SPI_CR = AT91C_SPI_SPIEN;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
unsigned int SPI_Xfer               /*  -  SPI   master */
    (
    unsigned int TransmitData,      /* ()  -  ( 16 )   */
    int Last                        /* ()  -   ,  NPCS   */
    )                               /*    (RD) */
    {
    if (f_mode != SPI_MODE_MASTER)
        return 0;

    /*  ,    (   
       MASTER   ) . */
    while (!(AT91C_BASE_SPI->SPI_SR & AT91C_SPI_TDRE));
    /*           NPCS
     * (DLYBCT = 1: 32 MCK   CLK   NPCS).
     *    ,      .
     * :         */
    if (Last)
        {
        AT91C_BASE_SPI->SPI_CSR[3] =
            (AT91C_BASE_SPI->SPI_CSR[3] & ~(AT91C_SPI_DLYBCT | AT91C_SPI_CSAAT)) |
            (AT91C_SPI_DLYBCT & (1 << 24));
        }
    else
        {
        AT91C_BASE_SPI->SPI_CSR[3] =
            (AT91C_BASE_SPI->SPI_CSR[3] & ~AT91C_SPI_DLYBCT) | AT91C_SPI_CSAAT;
        }
    /*   */
    AT91C_BASE_SPI->SPI_TDR = TransmitData & AT91C_SPI_TD;
    while (!(AT91C_BASE_SPI->SPI_SR & AT91C_SPI_RDRF));
    /*     */
    return (AT91C_BASE_SPI->SPI_RDR & AT91C_SPI_RD);
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
