#ifndef USB_STACKDESCR_H_
#define USB_STACKDESCR_H_

#define USB_ENDPOINT_DIRECTION_IN       0x80
#define USB_ENDPOINT_DIRECTION_OUT      0x00

#define USB_ENDPOINT_TYPE_CONTROL       0x00
#define USB_ENDPOINT_TYPE_ISOCHRONOUS   0x01
#define USB_ENDPOINT_TYPE_BULK          0x02
#define USB_ENDPOINT_TYPE_INTERRUPT     0x03

#define USB_DEVICE_DESCRIPTOR_TYPE                0x01
#define USB_CONFIGURATION_DESCRIPTOR_TYPE         0x02
#define USB_STRING_DESCRIPTOR_TYPE                0x03
#define USB_INTERFACE_DESCRIPTOR_TYPE             0x04
#define USB_ENDPOINT_DESCRIPTOR_TYPE              0x05
#define USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE      0x06
#define USB_OTHER_SPEED_CONF_DESCRIPTOR_TYPE      0x07
#define USB_POWER_DESCRIPTOR_TYPE                 0x08

/* Definitions for bits in the bmAttributes field of  */
/*   (Table 9-10) Standard Configuration Descriptor   */
/* Reserved (reset to zero) */                    /* D4...D0 */
#define USB_CONFIG_BUS_NOWAKEUP           0x80 /* D5=0 + D6=0 + D7=1 */
#define USB_CONFIG_SELF_NOWAKEUP          0xC0 /* D5=0 + D6=1 + D7=1 */
#define USB_CONFIG_BUS_WAKEUP             0xA0 /* D5=1 + D6=0 + D7=1 */
#define USB_CONFIG_SELF_WAKEUP            0xE0 /* D5=1 + D6=1 + D7=1 */
/* Reserved (set to one) */                       /*   D7    */

#define ENDPOINT_COUNT                            0x02

#pragma pack (push)
#pragma pack (1)

typedef struct _USB_DEVICE_DESCRIPTOR
{
  byte  bLength;
  byte  bDescriptorType;
  ushort  bcdUSB;
  byte  bDeviceClass;
  byte  bDeviceSubClass;
  byte  bDeviceProtocol;
  byte  bMaxPacketSize0;
  ushort  idVendor;
  ushort  idProduct;
  ushort  bcdDevice;
  byte  iManufacturer;
  byte  iProduct;
  byte  iSerialNumber;
  byte  bNumConfigurations;
} USB_DEVICE_DESCRIPTOR;

//   high speed
typedef struct _USB_DEVICE_QUALIFIER_DESCRIPTOR
{
    byte bLength;
    byte bDescriptorType;
    ushort bcdUSB;
    byte bDeviceClass;
    byte bDeviceSubClass;
    byte bDeviceProtocol;
    byte bMaxPacketSize0;
    byte bNumConfigurations;
    byte bReserved;
} USB_DEVICE_QUALIFIER_DESCRIPTOR;

typedef struct _USB_CONFIGURATION_DESCRIPTOR
{
  byte  bLength;
  byte  bDescriptorType;
  ushort  wTotalLength;
  byte  bNumInterfaces;
  byte  bConfigurationValue;
  byte  iConfiguration;
  byte  bmAttributes;
  byte  MaxPower;
} USB_CONFIGURATION_DESCRIPTOR;

//    hi-speed
typedef struct _USB_OTHER_SPEED_DESCRIPTOR
{
  byte  bLength;
  byte  bDescriptorType;
  ushort wTotalLength;
  byte bNumInterfaces;
  byte bConfigurationValue;
  byte iConfiguration;
  byte bmAttributes;
  byte bMaxPower;
} USB_OTHER_SPEED_DESCRIPTOR;

typedef struct _USB_INTERFACE_DESCRIPTOR
{
  byte bLength;
  byte bDescriptorType;
  byte bInterfaceNumber;
  byte bAlternateSetting;
  byte bNumEndpoints;
  byte bInterfaceClass;
  byte bInterfaceSubClass;
  byte bInterfaceProtocol;
  byte iInterface;
} USB_INTERFACE_DESCRIPTOR;

typedef struct _USB_ENDPOINT_DESCRIPTOR
{
  byte  bLength;
  byte  bDescriptorType;
  byte  bEndpointAddress;
  byte  bmAttributes;
  ushort  wMaxPacketSize;
  byte  bInterval;
} USB_ENDPOINT_DESCRIPTOR;

/*
typedef struct _USB_STRING_DESCRIPTOR
{
  byte  bLength;
  byte  bDescriptorType;
  ushort String[250];
} USB_STRING_DESCRIPTOR;
*/

typedef struct _FULL_CONFIG_DESCRIPTOR
{
  USB_CONFIGURATION_DESCRIPTOR Conf;
  USB_INTERFACE_DESCRIPTOR Interface;
  USB_ENDPOINT_DESCRIPTOR Endpoints[ENDPOINT_COUNT];
}FULL_CONFIG_DESCRIPTOR;

#pragma pack (pop)

/*================================================================================================*/
typedef struct
    {
    unsigned int Signature;
    unsigned int StringCount;   /*   ( ) */
    const USB_DEVICE_DESCRIPTOR *DeviceDescriptorPtr;
    const FULL_CONFIG_DESCRIPTOR *ConfigDescriptorPtr;
    const ushort *StrLangList;  /* USB string descriptor    */
    const ushort *const *String; /*    USB string descriptor',
                                     StringCount    */
    }
    t_USB_DescriptorDirectory;
/*================================================================================================*/

/*================================================================================================*/
extern const t_USB_DescriptorDirectory *g_DescDirPtr;

/*------------------------------------------------------------------------------------------------*/
int USB_LoadDescriptorDirectory
    (
    const t_USB_DescriptorDirectory *pDir
    );
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /* USB_STACKDESCR_H_ */
