/*================================================================================================*
 *   E-140
 *================================================================================================*/

#ifndef ADC_H_
#define ADC_H_

#include "USB_GlobDefs.h"

/*================================================================================================*/
#define ADC_MAX_CH          128                             /*     */
#define ADC_BUFFER_SIZE_B   (28 * 1024)                     /*      */
#define ADC_BLK_SIZE_B      (1 * IN_ENDPOINT_SIZE)          /*   DMA   */

#define ADC_BLK_SIZE_W  (ADC_BLK_SIZE_B / 2)                /*   DMA  16bit  */
#define ADC_NBLOCKS     (ADC_BUFFER_SIZE_B / ADC_BLK_SIZE_B)/*   DMA   */

#if ((ADC_BUFFER_SIZE_B % ADC_BLK_SIZE_B) != 0) || (ADC_NBLOCKS < 4)
#error "Invalid ADC buffer configuration"
#endif

/*    */
#define ADC_SYNC_MODE_MASK  0x0F
#define ADC_SYNC_SOFT       0       /*   ( ) */
#define ADC_SYNC_EXT_FRAME  1       /*      ( INT) */
#define ADC_SYNC_EXT_START  2       /*      ( INT) */
#define ADC_SYNC_LVL_ABOVE  3       /*       ,  */
#define ADC_SYNC_LVL_BELOW  4       /*       ,  */
#define ADC_SYNC_EDGE_RISE  5       /*      ,  */
#define ADC_SYNC_EDGE_FALL  6       /*      ,  */
#define ADC_SYNC_EXT_CLOCK  0x40    /*  : 0 = , 1 =  */
#define ADC_SYNC_ENABLE_OUT 0x80    /*     SYNC ( EXT_CLOCK = 0) */
/*================================================================================================*/

/*================================================================================================*/
/*   ( ,   ) */
#pragma pack(1)
typedef struct
    {
    unsigned char ChanCtl[ADC_MAX_CH]; /*     */
    unsigned short SampleRateDiv;   /*    (8000/Freq_kHz) */
    unsigned char NumChannelsUsed;  /*    */
    unsigned char InterFrameDelay;  /*   (   ), -1 */
    unsigned char SyncMode;         /*   */
    unsigned char SyncChanCtl;      /*      */
    short int SyncThreshold;        /*    ( ) */
    }
    t_ADC_ParamStruct;
#pragma pack()
/*================================================================================================*/

/*================================================================================================*/
/*    .   ,     
 *      USB,     .
 *     USB. */
extern int g_ADC_StopReceived;

/*       */
extern short int g_ADC_SingleResult;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void ADC_Init                       /*      SSC */
    (void);
/*------------------------------------------------------------------------------------------------*/
int ADC_IsRunning                   /* ,     (  ) */
    (void);
/*------------------------------------------------------------------------------------------------*/
void ADC_GetParams                  /*    ( ) */
    (
    t_ADC_ParamStruct *pParams
    );
/*------------------------------------------------------------------------------------------------*/
void ADC_SetParams                  /*    ( ) */
    (
    const t_ADC_ParamStruct *pParams
    );
/*------------------------------------------------------------------------------------------------*/
int ADC_Start                       /*      */
    (
    int SingleFrame                 /* ()  - 0 =  , 1 =   */
    );                              /*  1  , 0   */
/*  :
 *     SSC    DMA. SSC .
 *     .
 *           
 *     f_ADC_Params,        .
 */
/*------------------------------------------------------------------------------------------------*/
int ADC_StartSingleChannel          /*      g_ADC_SingleResult */
    (
    unsigned char ChanCtl           /* ()  -   */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int ADC_Stop                        /*   ,    */
    (void);                         /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
void ADC_Worker                     /*         USB */
    (void);
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /* ADC_H_ */
