/*================================================================================================*
 *    E-140
 *================================================================================================*/

#ifndef DAC_H_
#define DAC_H_

#include "USB_GlobDefs.h"

/*================================================================================================*/
#define DAC_BUFFER_SIZE_B  (20 * 1024)                      /*      */
#define DAC_BLK_SIZE_B     (4 * OUT_ENDPOINT_SIZE)          /*   DMA   */

#define DAC_BUFFER_SIZE_DW (DAC_BUFFER_SIZE_B / 4)          /*     32bit  */
#define DAC_BLK_SIZE_W     (DAC_BLK_SIZE_B / 2)             /*   DMA  16bit  */
#define DAC_BLK_SIZE_DW    (DAC_BLK_SIZE_B / 4)             /*   DMA  32bit  */
#define DAC_NBLOCKS        (DAC_BUFFER_SIZE_B / DAC_BLK_SIZE_B) /*   DMA   */

#define DAC_MIN_DMA_SAMPLES 8               /*       */

/*  ,   ,      
    DMA,    ,     (  ),
        .        
   ,     . */
#define DAC_BUFFER_SIZE_DW_TOTAL (DAC_BUFFER_SIZE_DW + DAC_MIN_DMA_SAMPLES)

#if ((DAC_BUFFER_SIZE_B % DAC_BLK_SIZE_B) != 0) || (DAC_NBLOCKS < 4)
#error "Invalid DAC buffer configuration"
#endif

/*  ,        .
 * (      ,   )
 */
#define DAC_DEFAULT_PRELOAD_SAMPLES  2048
#if (DAC_DEFAULT_PRELOAD_SAMPLES > DAC_BUFFER_SIZE_DW) \
    || (DAC_DEFAULT_PRELOAD_SAMPLES < DAC_BLK_SIZE_DW * 2) \
    || ((DAC_DEFAULT_PRELOAD_SAMPLES % DAC_BLK_SIZE_DW) != 0)
#error "Invalid DAC default preload size"
#endif
/*================================================================================================*/

/*================================================================================================*/
/*   ( ,   ) */

#define DAC_MODE_SET_CONST_ON_STOP  0x01    /*       
                                               params.StopState */
#define DAC_MODE_NO_CLEAR_BUFFER    0x02    /*      
                                                 TotalSamples    */
#define DAC_MODE_LOOP               0x10    /*   (0 =  ) */

#pragma pack(1)
typedef struct
    {
    unsigned short RateDiv;         /*  , f = 200000/(RateDiv+1) Hz */
    unsigned char SyncWithADC;      /* 0 =  ; 1 =     */
    unsigned char RunMode;          /*   ( DAC_MODE_...) */
    /*    3.06: */
    unsigned int TotalSamples;      /*   ( DWORD),   
                                          (   
                                       ).  0,  . */
    unsigned short PreloadSamples;  /* . :    DWORD,   
                                       . :     DWORD */
                                    /*  0,    DAC_DEFAULT_PRELOAD_SAMPLES. */
    unsigned short StopState[2];    /*  ( 2 )    ,
                                        (RunMode & DAC_MODE_SET_CONST_ON_STOP) != 0 */
    unsigned short StartOffset;     /*       */
    }
    t_DAC_ParamStruct;
#pragma pack()
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void DAC_Init                       /*    */
    (void);
/*------------------------------------------------------------------------------------------------*/
void DAC_GetParams                  /*    ( ) */
    (
    t_DAC_ParamStruct *pParams
    );
/*------------------------------------------------------------------------------------------------*/
int DAC_SetParams                   /*    ( ) */
    (
    const t_DAC_ParamStruct *pParams
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int DAC_SingleSample                /*     */
    (
    short int Ch0_Val,              /*   0 */
    short int Ch1_Val               /*   1 */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int DAC_StartStream                 /*      */
    (void);                         /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int DAC_Stop                        /*        */
    (
    unsigned int clear_buf          /* 0 =    */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
void DAC_Worker                     /*    */
    (void);
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /* DAC_H_ */
