/*================================================================================================*
 *    E-140
 *================================================================================================*/

#ifndef DAC_H_
#define DAC_H_

#include "USB_GlobDefsAsm.h"

/*================================================================================================*/
#define DAC_U32_SIZE        4       /*     (unsigned 32-bit) */
#define DAC_BUFFER_SIZE     8192                                /*      */
#define DAC_BLK_SIZE        (4 * OUT_ENDPOINT_SIZE)             /*   DMA */
#define DAC_BLK_NSAMPLES    (DAC_BLK_SIZE / DAC_U32_SIZE)       /*  ,  32-  */
#define DAC_NBLOCKS         (DAC_BUFFER_SIZE / DAC_BLK_SIZE)    /*     */

/*  ,        .
 * (      ,    )
 */
#define DAC_FULL_BLOCKS_BEFORE_START    DAC_NBLOCKS

#if DAC_NBLOCKS < 4
#error "Invalid DAC buffer configuration"
#endif
/*================================================================================================*/

/*================================================================================================*/
/*   ( ,   ) */
#pragma pack(1)
typedef struct
    {
    unsigned short RateDiv;         /*  , f = 200000/(RateDiv+1) Hz */
    unsigned char SyncWithADC;      /* 0 =  ; 1 =     */
    unsigned char SetZeroOnStop;    /* 1 =       0 */
    unsigned char Reserved[12];
    }
    t_DAC_ParamStruct;
#pragma pack()
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
void DAC_Init                       /*    */
    (void);
/*------------------------------------------------------------------------------------------------*/
void DAC_GetParams                  /*    ( ) */
    (
    t_DAC_ParamStruct *pParams
    );
/*------------------------------------------------------------------------------------------------*/
void DAC_SetParams                  /*    ( ) */
    (
    const t_DAC_ParamStruct *pParams
    );
/*------------------------------------------------------------------------------------------------*/
int DAC_SingleSample                /*     */
    (
    unsigned int DAC_Val            /*  16  =  0,  16  =  1 */
    );                              /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int DAC_StartStream                 /*      */
    (void);                         /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
int DAC_Stop                        /*        */
    (void);                         /*  1  , 0   */
/*------------------------------------------------------------------------------------------------*/
void DAC_Worker                     /*    */
    (void);
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/

#endif /* DAC_H_ */
