#include "globdefs.h"
#include "USBHardware.h"
#include "nvram.h"
#include "cpld.h"
#include "adc.h"
#include "dac.h"
#include "timers.h"
#include "wdt.h"
#include "pio.h"

/*================================================================================================*/
/*    callback' */
#define OP_ADC_STOP             0x10
#define OP_ADC_START_STREAM     0x11
#define OP_ADC_START_FRAME      0x12
#define OP_DAC_STOP             0x20
#define OP_DAC_START_STREAM     0x21
#define OP_DAC_SINGLE_OLD       0x22
#define OP_DAC_SINGLE           0x23
#define OP_GET_PARAM            0x30
#define OP_SET_PARAM            0x31

/*       (  VDRQ_...) */
#define DEVICE_TO_HOST()    \
    ((CurrentUSBRequest.bmRequestType & REQUEST_ANSWER_DIRECTION) == REQUEST_ANSWER_DIRECTION_DEVICE_TO_HOST)
#define HOST_TO_DEVICE()    \
    ((CurrentUSBRequest.bmRequestType & REQUEST_ANSWER_DIRECTION) != REQUEST_ANSWER_DIRECTION_DEVICE_TO_HOST)
/*================================================================================================*/

/*================================================================================================*/
/*      USB control channel */
void VDRQ_undefined(void);
void VDRQ_put_array(void);
void VDRQ_get_array(void);
void VDRQ_start_adc(void);
void VDRQ_stop_adc(void);
void VDRQ_start_dac(void);
void VDRQ_stop_dac(void);
void VDRQ_adc_single_frame(void);
void VDRQ_reset(void);
void VDRQ_get_module_name(void);
void VDRQ_bootloader_request(void);

/* ,      Device Request USB */
USB_request_Handler VendorDeviceRequest[VDRQ_COUNT] =
    {
    /* 00 */ VDRQ_undefined,
    /* 01 */ VDRQ_put_array,
    /* 02 */ VDRQ_get_array,
    /* 03 */ VDRQ_start_adc,
    /* 04 */ VDRQ_stop_adc,
    /* 05 */ VDRQ_adc_single_frame,
    /* 06 */ VDRQ_start_dac,
    /* 07 */ VDRQ_stop_dac,
    /* 08 */ VDRQ_undefined,
    /* 09 */ VDRQ_undefined,
    /* 0A */ VDRQ_reset,
    /* 0B */ VDRQ_get_module_name,
    /* 0C */ VDRQ_undefined,
    /* 0D */ VDRQ_undefined,
    /* 0E */ VDRQ_undefined,
    /* 0F */ VDRQ_bootloader_request
    };
/*================================================================================================*/

/*================================================================================================*/

/**************************************************************************************************
 ** CALLBACK'     USB control pipe                                    **
 **************************************************************************************************/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_ADC_Command
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /* Host to device,     (  ) */
    if (!SetLowPowerMode(0)) /*      */
        return -1;
    switch (UserParam)
        {
        case OP_ADC_START_STREAM:
            if (!ADC_Start(0)) return -1;
            break;
        case OP_ADC_START_FRAME:
            if (!ADC_Start(1)) return -1;
            break;
        case OP_ADC_STOP:
            if (!ADC_Stop()) return -1;
            break;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_ADC_Param
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < sizeof(t_ADC_ParamStruct)) return -1;
    switch (UserParam)
        {
        case OP_GET_PARAM:
            ADC_GetParams((void*)pBuffer);
            break;
        case OP_SET_PARAM:
            ADC_SetParams((void*)pBuffer);
            break;
        }
    return sizeof(t_ADC_ParamStruct);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_ADC_Single
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /* Host to device,   -   */
    if (RequestLen < 1) return -1;

    if (!SetLowPowerMode(0)) /*      */
        return -1;
    /*   */
    if (!ADC_StartSingleChannel(pBuffer[0]))
        return -1;
    /* ,     */
    TIMER_StartTimeout(100);
    while (ADC_IsRunning() && !TIMER_TimeoutExpired())
        {
        WDT_Restart();
        ADC_Worker();
        }
    if (ADC_IsRunning())
        { /* ,    */
        ADC_Stop();
        return -1;
        }
    /* ;    g_ADC_SingleResult */
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_DAC_Command
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /*  : , ,  */
    unsigned int data;
    static unsigned int last_dac = 0;
    if (!SetLowPowerMode(0)) /*      */
        return -1;
    switch (UserParam)
        {
        case OP_DAC_SINGLE_OLD:
            if (RequestLen < 2) return -1;
            data = ((unsigned int)pBuffer[1] << 8) | pBuffer[0];
#if 0
            //## test:   / 
            if (data)
                {
                t_DAC_ParamStruct dp;
                dp.RateDiv = data & 7;
                dp.SyncWithADC = ((data & 0x20) != 0);
                dp.SetZeroOnStop = ((data & 0x40) == 0);
                DAC_SetParams(&dp);
                if (!DAC_StartStream()) return -1;
                }
            else
                {
                if (!DAC_Stop()) return -1;
                }
#else
            /*      :
             *  - 16- .
             *  [11..0] = ;  12 =  .
             */
            data <<= 4;
            if (data & 0x10000)
                { /*   ,  1 */
                last_dac = (last_dac & 0x0000FFFF) | (data << 16);
                }
            else
                { /*   ,  0 */
                last_dac = (last_dac & 0xFFFF0000) | (data & 0xFFFF);
                }
            if (!DAC_SingleSample(last_dac)) return -1;
#endif
            break;
        case OP_DAC_SINGLE:
            /*      :
             *  - 32- .
             *  [15..0] =   0;
             *  [31..16] =   1;
             */
            if (RequestLen < 4) return -1;
            last_dac = ((unsigned int)pBuffer[3] << 24)
                     | ((unsigned int)pBuffer[2] << 16)
                     | ((unsigned int)pBuffer[1] << 8)
                     | pBuffer[0];
            if (!DAC_SingleSample(last_dac)) return -1;
            break;
        case OP_DAC_START_STREAM:
            /*    ,    DAC_SetParams() */
            if (!DAC_StartStream()) return -1;
            break;
        case OP_DAC_STOP:
            if (!DAC_Stop()) return -1;
            break;
        }
    return 0; /*   */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_DAC_Param
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < sizeof(t_DAC_ParamStruct)) return -1;
    switch (UserParam)
        {
        case OP_GET_PARAM:
            DAC_GetParams((void*)pBuffer);
            break;
        case OP_SET_PARAM:
            DAC_SetParams((void*)pBuffer);
            break;
        }
    return sizeof(t_DAC_ParamStruct);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_GetADCResult
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < 2) return -1;
    pBuffer[0] = g_ADC_SingleResult & 0xFF;
    pBuffer[1] = (g_ADC_SingleResult >> 8) & 0xFF;
    return 2;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_GetDACPresence
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < 1) return -1;
    pBuffer[0] = ((AT91C_BASE_PIOA->PIO_PDSR & PIO_N_DACSENSE) == 0);
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_ReadDescriptor
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /* UserParam =    */
    return (NVRAM_ReadDeviceDescriptor(UserParam, pBuffer, RequestLen)) ? (int)RequestLen : -1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_ReadNVRAM
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /* UserParam =     NVRAM */
    return (NVRAM_ReadUserArea(UserParam, pBuffer, RequestLen)) ? (int)RequestLen : -1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_ResetDevice
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    g_MainFlag |= MAIN_RESET_DEVICE;
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_SetNVRAMWriteFlag
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < 1) return -1;
    if (pBuffer[0] == 0)
        NVRAM_FlushBuffers(); /*   ,    */
    NVRAM_WriteEnable(pBuffer[0] != 0);
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_Suspend
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < 1) return -1;
    /* pBuffer[0] = suspend mode flag */
    return SetLowPowerMode(pBuffer[0] != 0) ? 1 : -1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_TTLDriverControl
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < 1) return -1;
    if (!SetLowPowerMode(0)) /*      */
        return -1;
    if (!CPLD_CFG_SetConfigBits(CPLD_CONFIG_TTLOUT_ENABLE,
            (pBuffer[0] ? CPLD_CONFIG_TTLOUT_ENABLE : 0)))
        return -1;
    return 1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_TTLIn
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    unsigned short int data;

    if (RequestLen < 2) return -1;
    if (!SetLowPowerMode(0)) /*      */
        return -1;
    if (!CPLD_TTL_In(&data))
        return -1;
    pBuffer[0] = data & 0xFF; /*   */
    pBuffer[1] = data >> 8; /*   */
    return 2;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_TTLOut
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    if (RequestLen < 2) return -1;
    if (!SetLowPowerMode(0)) /*      */
        return -1;
    if (!CPLD_TTL_Out(((unsigned short int)pBuffer[1] << 8) | pBuffer[0]))
        return -1;
    return 2;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_WriteDescriptor
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /* UserParam =    */
    return (NVRAM_WriteDeviceDescriptor(UserParam, pBuffer, RequestLen)) ? (int)RequestLen : -1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_WriteNVRAM
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    /* UserParam =     NVRAM */
    return (NVRAM_WriteUserArea(UserParam, pBuffer, RequestLen)) ? (int)RequestLen : -1;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
int cb_USB_DummyWrite
    (unsigned int UserParam, unsigned char *pBuffer, unsigned int RequestLen)
    {
    return RequestLen;
    }
/*------------------------------------------------------------------------------------------------*/

/**************************************************************************************************
 **  Vendor Device Request'                                                         **
 **************************************************************************************************/

/*------------------------------------------------------------------------------------------------*/
void VDRQ_undefined(void)
    {
    USBBG_RequestError();
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*     bootload,        bootloader */
void VDRQ_bootloader_request(void)
    {
    g_MainFlag |= MAIN_JUMP_TO_BOOTROM;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*       */
void VDRQ_adc_single_frame(void)
    {
    if (HOST_TO_DEVICE())
        {
        if (!USBBG_Begin(USBBG_WRITE, OP_ADC_START_FRAME,
                                    CurrentUSBRequest.wLength, cb_USB_ADC_Command))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*    */
void VDRQ_start_adc(void)
    {
    if (HOST_TO_DEVICE())
        {
        if (!USBBG_Begin(USBBG_WRITE, OP_ADC_START_STREAM,
                                    CurrentUSBRequest.wLength, cb_USB_ADC_Command))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*    */
void VDRQ_stop_adc(void)
    {
    if (HOST_TO_DEVICE())
        {
        g_ADC_StopReceived = 1; /*   */
        if (!USBBG_Begin(USBBG_WRITE, OP_ADC_STOP,
                                    CurrentUSBRequest.wLength, cb_USB_ADC_Command))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*    */
void VDRQ_start_dac(void)
    {
    if (HOST_TO_DEVICE())
        {
        if (!USBBG_Begin(USBBG_WRITE, OP_DAC_START_STREAM,
                                    CurrentUSBRequest.wLength, cb_USB_DAC_Command))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*    */
void VDRQ_stop_dac(void)
    {
    if (HOST_TO_DEVICE())
        {
        if (!USBBG_Begin(USBBG_WRITE, OP_DAC_STOP,
                                    CurrentUSBRequest.wLength, cb_USB_DAC_Command))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/* ,     (, )   .
    ,  Offset        1.
    ,   0.
*/
static int f_RequestInRange
    (
    unsigned int ReqStart,          /* ()  -    */
    unsigned int ReqLength,         /* ()  -   */
    unsigned int RangeStart,        /* ()  -    */
    unsigned int RangeLength,       /* ()  -   */
    unsigned int *pOffset           /* () -     */
    )
    {
    unsigned int ReqEnd = ReqStart + ReqLength;
    if ((ReqStart >= RangeStart) && (ReqEnd >= ReqStart) && (ReqEnd <= RangeStart + RangeLength))
        {
        if (pOffset)
            *pOffset = ReqStart - RangeStart;
        return 1;
        }
    return 0;
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/* GET ARRAY */
void VDRQ_get_array()
    {
    unsigned int Address = ((CurrentUSBRequest.wIndex & 0xFF) << 16) | CurrentUSBRequest.wValue;
    unsigned int offset;

    if (!DEVICE_TO_HOST())
        {
        USBBG_RequestError();
        return;
        }
    if (f_RequestInRange(Address, CurrentUSBRequest.wLength,
        GETA_DEVICE_DESCR, NVRAM_DEVDESC_SIZE, &offset))
        {
        if (!USBBG_Begin(USBBG_READ, offset, CurrentUSBRequest.wLength, cb_USB_ReadDescriptor))
            USBBG_RequestError();
        }
    else if (f_RequestInRange(Address, CurrentUSBRequest.wLength,
        GETA_NVRAM_START, NVRAM_USERAREA_SIZE, &offset))
        {
        if (!USBBG_Begin(USBBG_READ, offset, CurrentUSBRequest.wLength, cb_USB_ReadNVRAM))
            USBBG_RequestError();
        }
    else if (f_RequestInRange(Address, CurrentUSBRequest.wLength,
        GETA_TEST_RAM, USB_FIFO_IN_ARRAY_SIZE + USB_FIFO_OUT_ARRAY_SIZE, &offset))
        {
        if (!USBBG_BeginWithBuffer(USBBG_READ, 0, CurrentUSBRequest.wLength, NULL,
                                   (char*)USB_FIFO_ArrayDWORD + offset,
                                   CurrentUSBRequest.wLength))
            USBBG_RequestError();
        }
    else switch (Address)
        {
        case GETA_ADC_PARAMS:
            if (!USBBG_Begin(USBBG_READ, OP_GET_PARAM,
                                       CurrentUSBRequest.wLength, cb_USB_ADC_Param))
                USBBG_RequestError();
            break;
        case GETA_DAC_PARAMS:
            if (!USBBG_Begin(USBBG_READ, OP_GET_PARAM,
                                       CurrentUSBRequest.wLength, cb_USB_DAC_Param))
                USBBG_RequestError();
            break;
        case GETA_TTL_IN:
            if (!USBBG_Begin(USBBG_READ, 0, CurrentUSBRequest.wLength, cb_USB_TTLIn))
                USBBG_RequestError();
            break;
        case GETA_ADC_RESULT:
            if (!USBBG_Begin(USBBG_READ, 0, CurrentUSBRequest.wLength, cb_USB_GetADCResult))
                USBBG_RequestError();
            break;
        case GETA_VERSION:
            if (!USBBG_BeginWithBuffer(USBBG_READ, 0, CurrentUSBRequest.wLength, NULL,
                                       (void*)&VersionInfo, sizeof(VersionInfo)))
                USBBG_RequestError();
            break;
        case GETA_DAC_PRESENCE:
            if (!USBBG_Begin(USBBG_READ, 0, CurrentUSBRequest.wLength, cb_USB_GetDACPresence))
                USBBG_RequestError();
            break;
        default:
            USBBG_RequestError();
            break;
        } /* switch */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/* PUT ARRAY */
void VDRQ_put_array()
    {
    unsigned int Address = ((CurrentUSBRequest.wIndex & 0xFF) << 16) | CurrentUSBRequest.wValue;
    unsigned int offset;

    if (!HOST_TO_DEVICE())
        {
        USBBG_RequestError();
        return;
        }
    if (f_RequestInRange(Address, CurrentUSBRequest.wLength,
        PUTA_DEVICE_DESCR, NVRAM_DEVDESC_SIZE, &offset))
        {
        if (!USBBG_Begin(USBBG_WRITE, offset, CurrentUSBRequest.wLength, cb_USB_WriteDescriptor))
            USBBG_RequestError();
        }
    else if (f_RequestInRange(Address, CurrentUSBRequest.wLength,
        PUTA_NVRAM_START, NVRAM_USERAREA_SIZE, &offset))
        {
        if (!USBBG_Begin(USBBG_WRITE, offset, CurrentUSBRequest.wLength, cb_USB_WriteNVRAM))
            USBBG_RequestError();
        }
    else if (f_RequestInRange(Address, CurrentUSBRequest.wLength,
        PUTA_TEST_RAM, USB_FIFO_IN_ARRAY_SIZE + USB_FIFO_OUT_ARRAY_SIZE, &offset))
        {
        if (!USBBG_BeginWithBuffer(USBBG_WRITE, 0, CurrentUSBRequest.wLength, NULL,
                                   (char*)USB_FIFO_ArrayDWORD + offset,
                                   CurrentUSBRequest.wLength))
            USBBG_RequestError();
        }
    else switch (Address)
        {
        case PUTA_ADC_PARAMS:
            if (!USBBG_Begin(USBBG_WRITE, OP_SET_PARAM,
                                        CurrentUSBRequest.wLength, cb_USB_ADC_Param))
                USBBG_RequestError();
            break;
        case PUTA_DAC_PARAMS:
            if (!USBBG_Begin(USBBG_WRITE, OP_SET_PARAM,
                                        CurrentUSBRequest.wLength, cb_USB_DAC_Param))
                USBBG_RequestError();
            break;
        case PUTA_NVRAM_WENABLE:
            if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_SetNVRAMWriteFlag))
                USBBG_RequestError();
            break;
        case PUTA_TTL_OUT:
            if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_TTLOut))
                USBBG_RequestError();
            break;
        case PUTA_TTL_ENABLE:
            if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_TTLDriverControl))
                USBBG_RequestError();
            break;
        case PUTA_ADC_CHANNEL:
            if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_ADC_Single))
                USBBG_RequestError();
            break;
        case PUTA_DUMMY:
            if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_DummyWrite))
                USBBG_RequestError();
            break;
        case PUTA_DAC_OUT_OLD:
            if (!USBBG_Begin(USBBG_WRITE, OP_DAC_SINGLE_OLD,
                                        CurrentUSBRequest.wLength, cb_USB_DAC_Command))
                USBBG_RequestError();
            break;
        case PUTA_DAC_OUT:
            if (!USBBG_Begin(USBBG_WRITE, OP_DAC_SINGLE,
                                        CurrentUSBRequest.wLength, cb_USB_DAC_Command))
                USBBG_RequestError();
            break;
        case PUTA_SUSPEND:
            if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_Suspend))
                USBBG_RequestError();
            break;
        default:
            USBBG_RequestError();
            break;
        } /* switch */
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*    */
void VDRQ_get_module_name(void)
    {
    if (DEVICE_TO_HOST())
        {
        if (!USBBG_BeginWithBuffer(USBBG_READ, 0, CurrentUSBRequest.wLength, NULL,
                                   (void*)ModuleName, sizeof(ModuleName)))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*   */
void VDRQ_reset(void)
    {
    if (HOST_TO_DEVICE())
        {
        if (!USBBG_Begin(USBBG_WRITE, 0, CurrentUSBRequest.wLength, cb_USB_ResetDevice))
            USBBG_RequestError();
        }
    else
        {
        USBBG_RequestError();
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
