#include "GlobDefs.h"
#include "USBHardware.h"
#include "USBHard.h"

//__no_init
DWORD USB_FIFO_ArrayDWORD[(USB_FIFO_IN_ARRAY_SIZE + USB_FIFO_OUT_ARRAY_SIZE + 3)/4];

byte *const USB_OUT_Array = (byte*)USB_FIFO_ArrayDWORD;
const uint USB_OUT_Array_Size = USB_FIFO_OUT_ARRAY_SIZE;

byte *const USB_IN_Array = (byte*)USB_FIFO_ArrayDWORD + USB_FIFO_OUT_ARRAY_SIZE;
const uint USB_IN_Array_Size = USB_FIFO_IN_ARRAY_SIZE;

volatile int USB_Send_Bank_Full = 0;
volatile bool CurrentBank0 = true;
volatile bool USBConfigured = false;
volatile bool USBSuspend = false;

/*   */
unsigned char FifoOverflowFlag;
unsigned short int MaxFifoSizeUsed;

/*************************************************************************************
  ,    USB
**************************************************************************************/
#define USB_PULL_UP_PIN 0
void ContactUSB(bool Contact)
    {
    Contact &= 1;
    AT91C_BASE_PIOA->PIO_PER = 1 << USB_PULL_UP_PIN;
    AT91C_BASE_PIOA->PIO_OER = 1 << USB_PULL_UP_PIN;
    AT91C_BASE_PIOA->PIO_SODR = Contact << USB_PULL_UP_PIN;
    AT91C_BASE_PIOA->PIO_CODR = (!Contact) << USB_PULL_UP_PIN;
    USBConfigured = false;
    }

/****************************************************************************
 USB
******************************************************************************/
__thumb __ramfunc void ResetUSB()
{
    ResetEndpoints(0x0F); //   

    //   USB
    USB_Send_Bank_Full=0;
    CurrentBank0=true;

    FlushFIFOBuffers();
}

/*
 USB -    USB,
  reset endpoint   -      
*/
void ResetUSBSoftly(void)
{
    if (!(AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] &AT91C_UDP_TXPKTRDY))
    {
        //      -        -    -     -  
        ConfigureEndpoint(IN_ENDPOINT, BULK_IN, false);
        ConfigureEndpoint(IN_ENDPOINT, BULK_IN, true);
        USB_Send_Bank_Full=0;
    }

    ConfigureEndpoint(OUT_ENDPOINT, BULK_OUT, false);
    ConfigureEndpoint(OUT_ENDPOINT, BULK_OUT, true);
    CurrentBank0=true;

    FlushFIFOBuffers();
}


/*************************************************************************************
          USB
! 
 IN    
 OUT     
  
**************************************************************************************/
//__thumb __ramfunc
void FlushFIFOBuffers(void)
{
    USB_IN_ArrayPointer_Begin_Data=0;
    USB_IN_ArrayPointer_End_Data=0;
    USB_IN_ArrayCounter=0;
    USB_OUT_ArrayPointer_Begin_Data=0;
    USB_OUT_ArrayPointer_End_Data=0;
    USB_OUT_ArrayCounter=0;
}


/*
       USB
 
*/
bool WriteUSB_FIFO_IN(byte * pData, register int length)
{
    if (!length) return true;
    if ((USB_IN_ArrayCounter+length)>USB_IN_Array_Size) return false;

    USB_IN_ArrayCounter+=length;

    //           
    if ((USB_IN_ArrayPointer_End_Data+length)<USB_IN_Array_Size)
    {
        memcpy(&USB_IN_Array[USB_IN_ArrayPointer_End_Data],pData, length);
        USB_IN_ArrayPointer_End_Data+=length;
    }
    else
    {

        while (length--)
        {
            USB_IN_Array[USB_IN_ArrayPointer_End_Data++]=*pData++;
            if (USB_IN_ArrayPointer_End_Data>=USB_IN_Array_Size)USB_IN_ArrayPointer_End_Data=0;
        }
    }
    return true;
}

/*
       USB
     ,     
   .
 
*/
bool ReadUSB_FIFO_OUT(byte * pData, int length)
{
    if (USB_OUT_ArrayCounter<length) return false;

    //           
    if ((USB_OUT_ArrayPointer_Begin_Data+length)<USB_OUT_Array_Size)
    {
        memcpy(pData,&USB_OUT_Array[USB_OUT_ArrayPointer_Begin_Data],length); // memcpy -     
        USB_OUT_ArrayPointer_Begin_Data+=length;
    }
    else
    {
        register int Counts=length;
        while (Counts--)
        {
            *pData++=USB_OUT_Array[USB_OUT_ArrayPointer_Begin_Data++];
            if (USB_OUT_ArrayPointer_Begin_Data>=USB_OUT_Array_Size)USB_OUT_ArrayPointer_Begin_Data=0;
        }
    };

    USB_OUT_ArrayCounter-=length; // ,     FIFO

    if ((AT91C_BASE_UDP->UDP_IMR & (1 << OUT_ENDPOINT))==0)
    {
        //          
        Enable_USB_Interrupts(1<<OUT_ENDPOINT); //    -   Enable USB Interrupts
    };

    return true;
}


/*
  USB -   
     ,   FIFO,     
*/
bool ReadUSB(byte * pData, int length)
{
    bool Result = true;
    while (length&&Result)
    {
        int ReadData=0;
        while (!ReadData)
        {
            ReadData  = min(length, USB_OUT_ArrayCounter); // ,     
        }

        AT91F_disable_interrupt();
        Result=ReadUSB_FIFO_OUT(pData,ReadData);
        AT91F_enable_interrupt();

        pData+=ReadData;
        length-=ReadData;
    }
    return Result;
}


/*********************************************************************************************
    USB  
**********************************************************************************************/

/*
   USB  -   
      - ,      USB
  
*/

//__arm __ramfunc
void USBWriteBulkInterrupt(void)
{
    USB_Send_Bank_Full--; //    FIFO USB

    int cpt=min(USB_IN_ArrayCounter,IN_ENDPOINT_SIZE);

    if (USB_Send_Bank_Full==1) //     
    {
        //     - 
        AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] |= AT91C_UDP_TXPKTRDY; //      ,       
        if (cpt>0)
            Read_FIFO_IN_To_USB_FIFO_asm(cpt);
        return;
    }

    if (USB_Send_Bank_Full==0) //      
    {
        //            -  ping pong
        if (cpt>0)
        {
            //      
            Read_FIFO_IN_To_USB_FIFO_asm(cpt);

            AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] |= AT91C_UDP_TXPKTRDY; //      

            cpt=min(USB_IN_ArrayCounter,IN_ENDPOINT_SIZE);
            if (cpt>0)
            {
                //    -    TXPKTRDY    !!!!
                Read_FIFO_IN_To_USB_FIFO_asm(cpt);
            }
        }
    }
}


/*
   USB -   
*/
__arm __ramfunc  void USBReadBulkInterrupt(void)
{
    int PacketSize=GetNumberBytesFIFO(OUT_ENDPOINT);

    if (PacketSize>0)
    {
        //    

        if ((USB_OUT_ArrayCounter+PacketSize)>USB_OUT_Array_Size)
        {
            //   ,     USB -    USB FIFO - 
            AT91F_disable_interrupt(); //   ,     FIQ - UDP
            Disable_USB_Interrupts(1<<OUT_ENDPOINT); //    -   Enable USB Interrupts
            // AT91F_enable_interrupt();//  ,       -   ,
            //    FIQ -    ,   FIQ -  -   !!!!!!!!!!!!!!!
            return;
        }

        //    FIFO USB  FIFO 
        Write_FIFO_OUT_To_USB_FIFO_asm(PacketSize);
    }

    if (CurrentBank0)AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT]&=~AT91C_UDP_RX_DATA_BK0;
    else             AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT]&=~AT91C_UDP_RX_DATA_BK1;

    CurrentBank0=!CurrentBank0; //   
}

/*
  USB -   
*/
bool WriteUSB_FIFO(byte * pData, int length)
{
    if (!length) return true;
    if ((USB_IN_ArrayCounter+length)>USB_IN_Array_Size) return false;

    //    USB
    if (!USB_Send_Bank_Full)
    {
        //     FIFO  FIFO            !
        //  USB  ,     FIFO ARM
        register int cptThis=min(length,IN_ENDPOINT_SIZE);
        length-=cptThis;

        while (cptThis--)WriteFIFO(IN_ENDPOINT,*pData++);
        Udp_ep_set_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY); //     -      -      
        USB_Send_Bank_Full++;

        cptThis=min(length,IN_ENDPOINT_SIZE);
        if (cptThis>0)
        {
            //    -    TXPKTRDY    !!!!
            length-=cptThis;
            while (cptThis--)WriteFIFO(IN_ENDPOINT,*pData++);
            USB_Send_Bank_Full++;
        }
    }

    return WriteUSB_FIFO_IN(pData,length); //    FIFO
}

/*
  USB -   
     ,   FIFO,     
*/
bool WriteUSB(byte * pData, int length)
{
    bool Result = true;
    while (length&&Result)
    {
        int SendData=0;
        while(!SendData)
        {
            AT91F_disable_interrupt();
            if(USB_IN_ArrayCounter > MaxFifoSizeUsed)  MaxFifoSizeUsed=USB_IN_ArrayCounter;
            SendData  = min(length,USB_IN_Array_Size-USB_IN_ArrayCounter); // ,     
            if(SendData) break;
            AT91F_enable_interrupt();

            if(!SendData) { FifoOverflowFlag=1; return 0; } //  
        }

        Result=WriteUSB_FIFO(pData,SendData);
        AT91F_enable_interrupt();

        pData+=SendData;
        length-=SendData;
    }
    return 1;
}

/*------------------------------------------------------------------------------------------------*/
void ResetBulkEndpoint(unsigned int EndpointNumber)
    {
    /*  endpoint'    */
    switch (EndpointNumber)
        {
        case IN_ENDPOINT:
            __disable_interrupt();
            /*    USB */
            USB_IN_ArrayPointer_Begin_Data = 0;
            USB_IN_ArrayPointer_End_Data = 0;
            USB_IN_ArrayCounter = 0;
            /*  endpoint */
            AT91C_BASE_UDP->UDP_RSTEP = (1 << IN_ENDPOINT);
            /*    CSR   */
            if (AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] & AT91C_UDP_TXPKTRDY)
                {
                AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] = AT91C_UDP_EPTYPE_BULK_IN;
                Udp_ep_clr_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY);
                Udp_ep_set_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY);
                Udp_ep_clr_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY);
                }
            AT91C_BASE_UDP->UDP_RSTEP = 0;
            USB_Send_Bank_Full = 0;
            DisableEndpoint(IN_ENDPOINT);
            EnableEndpoint(IN_ENDPOINT);
            __enable_interrupt();
            break;

        case OUT_ENDPOINT:
            __disable_interrupt();
            /*    USB */
            USB_OUT_ArrayPointer_Begin_Data = 0;
            USB_OUT_ArrayPointer_End_Data = 0;
            USB_OUT_ArrayCounter = 0;
            /*  endpoint */
#if 0
            while (AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] & (AT91C_UDP_RX_DATA_BK0 | AT91C_UDP_RX_DATA_BK1))
                {
                unsigned int nbytes, bank;
                bank = (CurrentBank0) ? AT91C_UDP_RX_DATA_BK0 : AT91C_UDP_RX_DATA_BK1;
                nbytes = (AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] & AT91C_UDP_RXBYTECNT) >> 16;
                if (nbytes)
                    {
                    while (nbytes--)
                        {
                        if (AT91C_BASE_UDP->UDP_FDR[OUT_ENDPOINT]);
                        }
                    Udp_ep_clr_flag(OUT_ENDPOINT, bank);
                    CurrentBank0 = !CurrentBank0;
                    }
                }
#endif
#if 0
            for (int k = 0; k < 2; k++)
                {
                unsigned int nbytes, bank;
                bank = (CurrentBank0) ? AT91C_UDP_RX_DATA_BK0 : AT91C_UDP_RX_DATA_BK1;
                nbytes = (AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] & AT91C_UDP_RXBYTECNT) >> 16;
                if (nbytes)
                    {
                    while (nbytes--)
                        {
                        if (AT91C_BASE_UDP->UDP_FDR[OUT_ENDPOINT]);
                        }
                    Udp_ep_clr_flag(OUT_ENDPOINT, bank);
                    CurrentBank0 = !CurrentBank0;
                    }
                }
#endif
            AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] = AT91C_UDP_EPTYPE_BULK_OUT | AT91C_UDP_EPEDS;
            AT91C_BASE_UDP->UDP_RSTEP = (1 << OUT_ENDPOINT);
            AT91C_BASE_UDP->UDP_RSTEP = 0;
            //DisableEndpoint(OUT_ENDPOINT);
            //EnableEndpoint(OUT_ENDPOINT);
            if (!(AT91C_BASE_UDP->UDP_IMR & (1 << OUT_ENDPOINT)))
                Enable_USB_Interrupts(1 << OUT_ENDPOINT);
            __enable_interrupt();
            break;
        }
    }
/*------------------------------------------------------------------------------------------------*/
