#include "main.h"
#include "USBHardware.h"
#include "USBHard.h"
#include "buffers.h"

byte* const USB_OUT_Array = (byte*)g_Buf.USB_FIFO_OUT;
const uint USB_OUT_Array_Size = USB_FIFO_OUT_ARRAY_SIZE;

byte* const USB_IN_Array = (byte*)g_Buf.USB_FIFO_IN;
const uint USB_IN_Array_Size = USB_FIFO_IN_ARRAY_SIZE;

volatile int USB_Send_Bank_Full = 0;
volatile bool CurrentBank0 = true;
volatile bool USBConfigured = false;
volatile bool USBSuspend = false;

#define min(a, b)   (((a) < (b)) ? (a) : (b))

/*================================================================================================*/
static int USB_IN_ArrayPointer_Begin_Data;
static int USB_IN_ArrayPointer_End_Data;
static int USB_OUT_ArrayPointer_Begin_Data;
static int USB_OUT_ArrayPointer_End_Data;
int USB_IN_ArrayCounter;
int USB_OUT_ArrayCounter;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
/*      .  Duff's Device */
#pragma optimize = no_inline
__arm __ramfunc  void reg2mem
    (unsigned char* mem, AT91_REG* reg, unsigned int count)
    {
    if (!count) return;
    unsigned int n = (count + 7) / 8;
    switch (count % 8)
        {
        case 0:	do{	*mem++ = *reg;
        case 7:		*mem++ = *reg;
        case 6:		*mem++ = *reg;
        case 5:		*mem++ = *reg;
        case 4:		*mem++ = *reg;
        case 3:		*mem++ = *reg;
        case 2:		*mem++ = *reg;
        case 1:		*mem++ = *reg;
            }while (--n);
        }
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
/*      .  Duff's Device */
#pragma optimize = no_inline
__arm __ramfunc  void mem2reg
    (AT91_REG* reg, const unsigned char* mem, unsigned int count)
    {
    if (!count) return;
    unsigned int n = (count + 7) / 8;
    switch (count % 8)
        {
        case 0:	do{	*reg = *mem++;
        case 7:		*reg = *mem++;
        case 6:		*reg = *mem++;
        case 5:		*reg = *mem++;
        case 4:		*reg = *mem++;
        case 3:		*reg = *mem++;
        case 2:		*reg = *mem++;
        case 1:		*reg = *mem++;
            }while (--n);
        }
    }
/*------------------------------------------------------------------------------------------------*/


/*************************************************************************************
  ,    USB
**************************************************************************************/
#define USB_PULL_UP_PIN 0
void ContactUSB(bool Contact)
    {
    Contact &= 1;
    AT91C_BASE_PIOA->PIO_PER = 1 << USB_PULL_UP_PIN;
    AT91C_BASE_PIOA->PIO_OER = 1 << USB_PULL_UP_PIN;
    AT91C_BASE_PIOA->PIO_SODR = Contact << USB_PULL_UP_PIN;
    AT91C_BASE_PIOA->PIO_CODR = (!Contact) << USB_PULL_UP_PIN;
    USBConfigured = false;
    }

/****************************************************************************
 USB
******************************************************************************/
__thumb __ramfunc void ResetUSB()
    {
    ResetEndpoints(0x0F); //   

    //   USB
    USB_Send_Bank_Full = 0;
    CurrentBank0 = true;

    FlushFIFOBuffers();
    }

/*
 USB -    USB,
  reset endpoint   -      
*/
void ResetUSBSoftly(void)
    {
    if (!(AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] & AT91C_UDP_TXPKTRDY))
        {
        //      -        -    -     -  
        ConfigureEndpoint(IN_ENDPOINT, BULK_IN, false);
        ConfigureEndpoint(IN_ENDPOINT, BULK_IN, true);
        USB_Send_Bank_Full = 0;
        }

    ConfigureEndpoint(OUT_ENDPOINT, BULK_OUT, false);
    ConfigureEndpoint(OUT_ENDPOINT, BULK_OUT, true);
    CurrentBank0 = true;

    FlushFIFOBuffers();
    }


/*************************************************************************************
          USB
! 
 IN    
 OUT     
  
**************************************************************************************/
//__thumb __ramfunc
void FlushFIFOBuffers(void)
    {
    USB_IN_ArrayPointer_Begin_Data = 0;
    USB_IN_ArrayPointer_End_Data = 0;
    USB_IN_ArrayCounter = 0;
    USB_OUT_ArrayPointer_Begin_Data = 0;
    USB_OUT_ArrayPointer_End_Data = 0;
    USB_OUT_ArrayCounter = 0;
    }


/*
       USB
 
*/
bool WriteUSB_FIFO_IN(byte * pData, register int length)
    {
    register int len1;
    if (length <= 0)
        return true;
    if ((USB_IN_ArrayCounter + length) > USB_IN_Array_Size)
        return false;

    USB_IN_ArrayCounter += length;

    len1 = min(length, USB_IN_Array_Size - USB_IN_ArrayPointer_End_Data);
    if (len1 > 0)
        memcpy(&USB_IN_Array[USB_IN_ArrayPointer_End_Data], pData, len1);
    if (length > len1)
        memcpy(&USB_IN_Array[0], pData, length - len1);

    USB_IN_ArrayPointer_End_Data += length;
    if (USB_IN_ArrayPointer_End_Data >= USB_IN_Array_Size)
        USB_IN_ArrayPointer_End_Data -= USB_IN_Array_Size;

    return true;
    }

/*
       USB
     ,     
   .
 
*/
bool ReadUSB_FIFO_OUT(byte * pData, int length)
    {
    register int len1;
    if (USB_OUT_ArrayCounter < length)
        return false;

    len1 = min(length, USB_OUT_Array_Size - USB_OUT_ArrayPointer_Begin_Data);
    if (len1 > 0)
        memcpy(pData, &USB_OUT_Array[USB_OUT_ArrayPointer_Begin_Data], len1);
    if (length > len1)
        memcpy(pData, &USB_OUT_Array[0], length - len1);

    USB_OUT_ArrayPointer_Begin_Data += length;
    if (USB_OUT_ArrayPointer_Begin_Data >= USB_OUT_Array_Size)
        USB_OUT_ArrayPointer_Begin_Data -= USB_OUT_Array_Size;

    USB_OUT_ArrayCounter -= length;

    //       ,   
    if (!(AT91C_BASE_UDP->UDP_IMR & (1 << OUT_ENDPOINT)))
        Enable_USB_Interrupts(1 << OUT_ENDPOINT);

    return true;
    }


/*
  USB -   
*/
bool ReadUSB(byte * pData, int length)
    {
    bool Result = true;
    while ((length > 0) && Result)
        {
        int ReadData;
        ReadData  = min(length, USB_OUT_ArrayCounter);
        if (ReadData)
            {
            USB_BEGIN_CRITICAL(1);
            Result = ReadUSB_FIFO_OUT(pData, ReadData);
            USB_END_CRITICAL(1);
            }
        else
            {
            Result = false;
            }

        pData += ReadData;
        length -= ReadData;
        }
    return Result;
    }


/*********************************************************************************************
    USB  
**********************************************************************************************/

/******************************************************************************************************
   FIFO     USB   
  */
__arm __ramfunc void Read_FIFO_IN_To_USB_FIFO(int length)
    {
    register unsigned int len1;
    register AT91_REG* pReg = &AT91C_BASE_UDP->UDP_FDR[IN_ENDPOINT];
    if (length <= 0) return;

    len1 = min(length, USB_IN_Array_Size - USB_IN_ArrayPointer_Begin_Data);
    if (len1 > 0)
        mem2reg(pReg, &USB_IN_Array[USB_IN_ArrayPointer_Begin_Data], len1);
    if (length > len1)
        mem2reg(pReg, &USB_IN_Array[0], length - len1);

    USB_IN_ArrayPointer_Begin_Data += length;
    if (USB_IN_ArrayPointer_Begin_Data >= USB_IN_Array_Size)
        USB_IN_ArrayPointer_Begin_Data -= USB_IN_Array_Size;

    USB_IN_ArrayCounter -= length;

    USB_Send_Bank_Full++;
    }

/******************************************************************************************************
   USB FIFO  FIFO 
  */
__arm __ramfunc void Write_FIFO_OUT_To_USB_FIFO(int length)
    {
    register unsigned int len1;
    register AT91_REG* pReg = &AT91C_BASE_UDP->UDP_FDR[OUT_ENDPOINT];
    if (length <= 0) return;

    USB_OUT_ArrayCounter += length;

    len1 = min(length, USB_OUT_Array_Size - USB_OUT_ArrayPointer_End_Data);
    if (len1 > 0)
        reg2mem(&USB_OUT_Array[USB_OUT_ArrayPointer_End_Data], pReg, len1);
    if (length > len1)
        reg2mem(&USB_OUT_Array[0], pReg, length - len1);

    USB_OUT_ArrayPointer_End_Data += length;
    if (USB_OUT_ArrayPointer_End_Data >= USB_OUT_Array_Size)
        USB_OUT_ArrayPointer_End_Data -= USB_OUT_Array_Size;
    }


/*
  USB  -   
      - ,      USB
*/

__arm __ramfunc
void USBWriteBulkInterrupt(void)
    {
    USB_Send_Bank_Full--; //    FIFO USB

    int cpt = min(USB_IN_ArrayCounter, IN_ENDPOINT_SIZE);

    if (USB_Send_Bank_Full == 1) //     
        {
        //     - 
        AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] |= AT91C_UDP_TXPKTRDY; //      ,       
        Read_FIFO_IN_To_USB_FIFO(cpt);
        return;
        }

    if (USB_Send_Bank_Full==0) //      
        {
        //            -  ping pong
        if (cpt > 0)
            {
            //      
            Read_FIFO_IN_To_USB_FIFO(cpt);
            AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] |= AT91C_UDP_TXPKTRDY; //      
            cpt = min(USB_IN_ArrayCounter, IN_ENDPOINT_SIZE);
            //    -    TXPKTRDY    
            Read_FIFO_IN_To_USB_FIFO(cpt);
            }
        }
    }


/*
   USB -   
*/
__arm __ramfunc  void USBReadBulkInterrupt(void)
{
    int PacketSize=GetNumberBytesFIFO(OUT_ENDPOINT);

    if (PacketSize>0)
    {
        //    

        if ((USB_OUT_ArrayCounter+PacketSize)>USB_OUT_Array_Size)
        {
            //   ,    USB -    USB FIFO - 
            Disable_USB_Interrupts(1<<OUT_ENDPOINT);
            return;
        }

        //    FIFO USB  FIFO 
        Write_FIFO_OUT_To_USB_FIFO(PacketSize);
    }

    if (CurrentBank0)AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT]&=~AT91C_UDP_RX_DATA_BK0;
    else             AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT]&=~AT91C_UDP_RX_DATA_BK1;

    CurrentBank0 = !CurrentBank0; //   
}

/*
  USB -   
*/
bool WriteUSB_FIFO(byte * pData, int length)
{
    if (!length) return true;
    if ((USB_IN_ArrayCounter+length)>USB_IN_Array_Size) return false;

    //    USB
    if (!USB_Send_Bank_Full)
        {
        //     FIFO  FIFO            !
        //  USB  ,     FIFO ARM
        register int cptThis = min(length, IN_ENDPOINT_SIZE);
        length -= cptThis;
        mem2reg(&AT91C_BASE_UDP->UDP_FDR[IN_ENDPOINT], pData, cptThis);
        pData += cptThis;

        Udp_ep_set_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY); //     -      -      
        USB_Send_Bank_Full++;

        cptThis = min(length, IN_ENDPOINT_SIZE);
        if (cptThis > 0)
            {
            //    -    TXPKTRDY    !!!!
            length -= cptThis;
            mem2reg(&AT91C_BASE_UDP->UDP_FDR[IN_ENDPOINT], pData, cptThis);
            pData += cptThis;
            USB_Send_Bank_Full++;
            }
        }

    return WriteUSB_FIFO_IN(pData, length); //    FIFO
    }

/*
  USB -   
    ,  
*/
bool WriteUSB(byte * pData, int length)
    {
    bool Result = true;
    while ((length > 0) && Result)
        {
        int SendData;
        USB_BEGIN_CRITICAL(1);
        /*
        if (USB_IN_ArrayCounter > g_Diag.USB_IN_FIFO_HighWatermark)
            g_Diag.USB_IN_FIFO_HighWatermark = USB_IN_ArrayCounter;
        */
        // ,     
        SendData = min(length, USB_IN_Array_Size - USB_IN_ArrayCounter);
        Result = (SendData) ? WriteUSB_FIFO(pData, SendData) : false;
        USB_END_CRITICAL(1);

        pData += SendData;
        length -= SendData;
        }
    return Result;
    }

/*------------------------------------------------------------------------------------------------*/
void ResetBulkEndpoint(unsigned int EndpointNumber)
    {
    /*  endpoint'    */
    switch (EndpointNumber)
        {
        case IN_ENDPOINT:
            USB_BEGIN_CRITICAL(1);
            /*    USB */
            USB_IN_ArrayPointer_Begin_Data = 0;
            USB_IN_ArrayPointer_End_Data = 0;
            USB_IN_ArrayCounter = 0;
            /*  endpoint */
            AT91C_BASE_UDP->UDP_RSTEP = (1 << IN_ENDPOINT);
            /*    CSR   */
            if (AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] & AT91C_UDP_TXPKTRDY)
                {
                AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] = AT91C_UDP_EPTYPE_BULK_IN;
                Udp_ep_clr_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY);
                Udp_ep_set_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY);
                Udp_ep_clr_flag(IN_ENDPOINT, AT91C_UDP_TXPKTRDY);
                }
            AT91C_BASE_UDP->UDP_RSTEP = 0;
            USB_Send_Bank_Full = 0;
            DisableEndpoint(IN_ENDPOINT);
            EnableEndpoint(IN_ENDPOINT);
            USB_END_CRITICAL(1);
            break;

        case OUT_ENDPOINT:
            USB_BEGIN_CRITICAL(1);
            /*    USB */
            USB_OUT_ArrayPointer_Begin_Data = 0;
            USB_OUT_ArrayPointer_End_Data = 0;
            USB_OUT_ArrayCounter = 0;
            /*  endpoint */
#if 0
            while (AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] & (AT91C_UDP_RX_DATA_BK0 | AT91C_UDP_RX_DATA_BK1))
                {
                unsigned int nbytes, bank;
                bank = (CurrentBank0) ? AT91C_UDP_RX_DATA_BK0 : AT91C_UDP_RX_DATA_BK1;
                nbytes = (AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] & AT91C_UDP_RXBYTECNT) >> 16;
                if (nbytes)
                    {
                    while (nbytes--)
                        {
                        if (AT91C_BASE_UDP->UDP_FDR[OUT_ENDPOINT]);
                        }
                    Udp_ep_clr_flag(OUT_ENDPOINT, bank);
                    CurrentBank0 = !CurrentBank0;
                    }
                }
#endif
#if 0
            for (int k = 0; k < 2; k++)
                {
                unsigned int nbytes, bank;
                bank = (CurrentBank0) ? AT91C_UDP_RX_DATA_BK0 : AT91C_UDP_RX_DATA_BK1;
                nbytes = (AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] & AT91C_UDP_RXBYTECNT) >> 16;
                if (nbytes)
                    {
                    while (nbytes--)
                        {
                        if (AT91C_BASE_UDP->UDP_FDR[OUT_ENDPOINT]);
                        }
                    Udp_ep_clr_flag(OUT_ENDPOINT, bank);
                    CurrentBank0 = !CurrentBank0;
                    }
                }
#endif
            AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] = AT91C_UDP_EPTYPE_BULK_OUT | AT91C_UDP_EPEDS;
            AT91C_BASE_UDP->UDP_RSTEP = (1 << OUT_ENDPOINT);
            AT91C_BASE_UDP->UDP_RSTEP = 0;
            //DisableEndpoint(OUT_ENDPOINT);
            //EnableEndpoint(OUT_ENDPOINT);
            if (!(AT91C_BASE_UDP->UDP_IMR & (1 << OUT_ENDPOINT)))
                Enable_USB_Interrupts(1 << OUT_ENDPOINT);
            USB_END_CRITICAL(1);
            break;
        }
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
