#include "USBHardware.h"
#include "timers.h"

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
/*   USB */
__arm __ramfunc void USB_Interrupt_Handler
    (void)
    {
    unsigned int status;
#if DEBUG_LED_USB_INTERRUPT
    DEBUG_LED_ON();
#endif
    g_TIMER_BlinkFlag = 1;
    status = AT91C_BASE_UDP->UDP_ISR;
    status &= (AT91C_BASE_UDP->UDP_IMR | AT91C_UDP_ENDBUSRES);

    if (status & (1 << IN_ENDPOINT))
        { /* BULK_IN -     USB */
        register unsigned int csr = AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT];
        if (csr & AT91C_UDP_TXCOMP)
            {
            AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] = csr & ~AT91C_UDP_TXCOMP;
            USBWriteBulkInterrupt();
            }
        else
            {
            AT91C_BASE_UDP->UDP_CSR[IN_ENDPOINT] = csr &
                ~(AT91C_UDP_RX_DATA_BK0 | AT91C_UDP_RX_DATA_BK1 | AT91C_UDP_ISOERROR);
            }
        }

    if (status & (1 << OUT_ENDPOINT))
        { /* BULK_OUT -   USB   */
        register unsigned int csr = AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT];
        if (csr & (AT91C_UDP_RX_DATA_BK0 | AT91C_UDP_RX_DATA_BK1))
            {
            USBReadBulkInterrupt();
            }
        else
            { /*  ,    */
            AT91C_BASE_UDP->UDP_CSR[OUT_ENDPOINT] = csr & ~(AT91C_UDP_TXCOMP | AT91C_UDP_ISOERROR);
            }
        }

    status &= ~((1 << IN_ENDPOINT) | (1 << OUT_ENDPOINT));
    if (status)
        {
        if (status & AT91C_UDP_ENDBUSRES)
            EndBusResProcessing();

        if (status & (1 << CONTROL_ENDPOINT))
            {
            register unsigned int csr = AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT];
            if (csr & AT91C_UDP_RXSETUP)
                {
                ControlCommandProcessing();
                }
            else
                { /*   ,    */
                //  if(!BackgroundControlOn)
                AT91C_BASE_UDP->UDP_CSR[CONTROL_ENDPOINT] = csr &
                    ~(AT91C_UDP_TXCOMP | AT91C_UDP_RX_DATA_BK0 | AT91C_UDP_RX_DATA_BK1 | AT91C_UDP_ISOERROR);
                }
            }

        if (status & AT91C_UDP_RXRSM)
            ExitSleepMode();
        if (status & AT91C_UDP_RXSUSP)
            EnterSleepMode();

        Clear_Usb_Interrupts(status);
        }
#if DEBUG_LED_USB_INTERRUPT
    DEBUG_LED_OFF();
#endif
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
